%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% Written (W) 2005-2009 Gunnar Raetsch, Gabriele Schweikert, Jonas Behr, Soeren Sonnenburg, Alexander Zien, Georg Zeller, Andre Noll, Cheng Soon Ong, Petra Philips
% Copyright (C) 2005-2009 Max Planck Society and Fraunhofer Institute FIRST 
%
function mgene_anno2signallabel(genome_dir, anno_dir, spf_dir, signal_name, covered_genic_positions_only) ;
% mgene_anno2signallabel(genome_dir, anno_dir, spf_dir, signal_name, covered_genic_positions_only) ;

global MGENE_TMP_PATH

if nargin<4,
  first_arg=[] ;
  if nargin>=1, first_arg=genome_dir; end ;
 
  if show_usage('mgene_anno2signallabel', '<genome-dir>, <annotation-dir>, <SPF-file-or-dir>, <signal-name> [, <genic-positions-flag>]', ...
     nargin, first_arg),
     return
  end ;
end ;

tmpname = tempname(MGENE_TMP_PATH) ;
if (~isempty(spf_dir) && spf_dir(end)=='/') || (length(spf_dir)>=5 && isequal(spf_dir(end-3:end), '.bspf'))
  spf_fname = tmpname ;
else		
  spf_fname = spf_dir ;
  spf_dir = tmpname ;
end ;
if ~exist('covered_genic_positions_only', 'var')
  covered_genic_positions_only = 1 ;
end ;

anno2signallabel('', anno_dir, genome_dir, spf_fname, spf_dir, signal_name, covered_genic_positions_only) ;

system(sprintf('rm -rf %s', tmpname)) ;

return

function anno2signallabel(anno_fname, anno_dir,genome_dir, spf_fname, spf_dir, signal_name, covered_genic_positions_only) ;
% anno2label(anno_fname, anno_dir,genome_dir, spf_fname, spf_dir, signal_name, covered_genic_positions_only) ;

if ~exist('covered_genic_positions_only'),
  covered_genic_positions_only = 1 ;
end ;

disp('------------------------------------') ;
disp('Step 1: Loading genome annotation...') ;
disp('------------------------------------') ;

genes = load_struct(sprintf('%s/genes.mat', anno_dir), 'genes') ;
fprintf('Loaded annotation with %i genes\n', length(genes)) ;

disp('-------------------------------------') ;
disp('Step 2: Loading genome information...') ;
disp('-------------------------------------') ;

genome_config = [genome_dir '/genome.config'] ;
genome_info = init_genome(genome_config);


if length(genes)==0 
  error('There are no genes in your annotation gene structure (AGS). Probably something went wrong during parsing');
end ;

% genes = filter_invalid_genes(genes,genome_info);

genes2label_helper(genes, genome_info, genome_config, spf_fname, spf_dir, signal_name, covered_genic_positions_only) ;

return

function genes2label_helper(genes, genome_info, genome_config, spf_fname, spf_dir, signal_name, covered_genic_positions_only) ;
% genes2label_helper(genes, genome_info, genome_config, spf_fname, spf_dir, signal_name, covered_genic_positions_only) ;

disp('-------------------------------------------------------') ;
disp('Step 5: Extracting labeled positions from the genome...') ;
disp('-------------------------------------------------------') ;

if ~isempty(spf_dir),
  galaxy_PAR.dir = spf_dir ;
else
  galaxy_PAR.dir = tempname ;
end ;
galaxy_PAR.genome_config = genome_config ;
PAR = create_PAR('exp_galaxy', galaxy_PAR);
PAR.Signal_name = signal_name ;
PAR.Signals.(signal_name).Conf_names = {} ;


if ~isempty(spf_dir),
  out_dir = [spf_dir '/tmp'] ;
else
  out_dir = tempname ;
end ;
unix(sprintf('mkdir -p %s', out_dir)) ;

contig_names = genome_info.contig_names ;
for c=1:length(contig_names)
  for s='+-',
    cs_idx = find(ismember({genes.chr}, contig_names{c}) & [genes.strand]==s) ;
    cs_genes = genes(cs_idx) ;

    if covered_genic_positions_only,
      covered_genic_positions = {} ;
      for i=1:length(cs_genes),
        covered_genic_positions{end+1} = cs_genes(i).start-1:cs_genes(i).stop+1 ;
      end ;
      covered_genic_positions=[covered_genic_positions{:}] ;
    end ;

    % get the list of positively labeled positions (those in the annotation)
    seq = load_genomic(contig_names{c}, s, 1, inf, genome_info) ;

    P_=struct ;
    P_.genes = cs_genes;
    P_.seq = seq;
    P_.Signal = PAR.Signals.(PAR.Signal_name);
    P_.info = PAR.info_genes.(PAR.Signal_name);
    P_.signal_name = PAR.Signal_name ;
    P_.strand = s; 
    [pos, label] = feval(PAR.Signals.(signal_name).label_fct, P_);
    clear P_ ;

    if covered_genic_positions_only,
      [tmp,idx1,idx2]=intersect(covered_genic_positions, pos) ;
      pos   = pos(idx2) ;
      label = label(idx2) ;
    end ;

    
    %subsample negative examples
    pos_idx = find(label==1);
    neg_idx = find(label==-1);
    if length(pos_idx)>0 && length(neg_idx)>0
      real_ratio = length(pos_idx)/length(neg_idx) ;
      fprintf('current class ratio (pos/neg): %2.1f%%\n', real_ratio) ;
      if (PAR.Signals.(signal_name).sampling.pos_neg_ratio - real_ratio) > 0
        % if real_ratio is smaller than pos_neg_ratio, there are to many negative examples
        rand('state', 24593854);
        keep = randperm(length(neg_idx));
        neg_idx = neg_idx(keep(1:floor(length(pos_idx) / PAR.Signals.(signal_name).sampling.pos_neg_ratio)));
        pos = pos([pos_idx neg_idx]);
        label = label([pos_idx neg_idx]);
        real_ratio = length(pos_idx)/length(neg_idx);
        fprintf('class ratio (pos/neg) after downsampling: %2.1f%%\n', real_ratio) ;
      end 
    end

    % sort the positions
    [tmp,idx] = sort(pos) ;
    pos = pos(idx) ;
    label = label(idx) ;
    
    % save them to a file
    filename_prefix = sprintf('%s/contig_%i%c', out_dir, c, s) ;
    fn_in=[filename_prefix '_label.spf'];
    unix(sprintf('rm -f %s', fn_in)) ;
    save_predictions_as_sigpred(filename_prefix, pos, label, 'label', signal_name, contig_names{c}, s, 0) ;
    save_predictions_as_sigpred_bin(filename_prefix, pos, label, 'label', signal_name, contig_names{c}, s) ;
  end ;
end ;
fprintf('Done.\n\n') ;

disp('----------------------------------') ;
disp('Step 6: Generating output files...') ;
disp('----------------------------------') ;
unix(sprintf('rm -f %s', spf_fname)) ;
cnt = 0 ;
for c = 1:length(contig_names)
  for s = '+-'
    P.chrom = c;
    P.strand = s;
    filename = sprintf('%s/contig_%i%s', out_dir, c, s);
    fn_in = [filename '_label.spf'];
    ret = unix(['cat ' fn_in ' >> ' spf_fname '; rm -f ' fn_in]) ;
    assert(ret==0) ;

    fn_mat = sprintf('%s/contig_%i%s_label_spf.mat', out_dir, c, s);
    fnall_mat = sprintf('%s/label_spf.mat', spf_dir);
    SPF=load(fn_mat);
    %fieldnames(SPF)

    save_append(fnall_mat, 1, sprintf('SPF_%i', cnt), SPF) ;
    cnt = cnt + 1 ;
    ret = unix(['rm -f ' fn_mat]) ;
    assert(ret==0) ;
  end ;
end ;
if length(spf_fname)>=3 && isequal(spf_fname(end-2:end), '.gz'),
  unix(sprintf('mv %s %s; gzip %s', spf_fname, spf_fname(1:end-3), spf_fname(1:end-3))) ;
end ;

fprintf('Done.\n\n') ;
unix(sprintf('rmdir %s', out_dir)) ;

return

function f=bigunique(s) ;
% f=bigunique(s) ;

f=[] ; step=ceil(length(s)/10) ;
for i=1:10,
  idx=1+(i-1)*step:min(length(s), i*step) ;
  f=unique([f unique(s(idx))]) ;
end ;


return

function genome_info=init_genome(config_fname)
% genome_info=init_genome(config_fname)  
%
% global g_genome_info ;

global g_genome_info genome_strings;

genome_strings = [];
g_est_fnames={} ;
g_prot_fnames={} ;
g_cdna_fnames={} ;
g_flcdna_fnames={} ;
g_contig_names={} ;
g_flat_fnames={} ;
g_fasta_fnames={} ;
g_annotation_fnames={} ;
g_alphabet='acgt' ;
g_basedir='' ;
g_max_intron_len=1000000 ;
g_min_intron_len=10 ;
g_merge_est_transcripts=1 ;

fd=fopen(config_fname, 'r') ;
while ~feof(fd),
  line = fgetl(fd) ;
  if ~ischar(line), break ; end ;
  if isempty(line), continue ; end ;
  if line(1)=='#', continue ; end ;
  if length(line)>8 && isequal(line(1:7),'BASEDIR')
    g_basedir=deblank(line(9:end)) ;
  elseif length(line)>21 && isequal(line(1:21), 'MERGE_EST_TRANSCRIPTS')
    g_merge_est_transcripts = str2num(line(22:end)) ; 
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_QGAP')
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_TGAP')
  elseif length(line)>17 && isequal(line(1:17),'MAX_CDNA_DELETION')
  elseif length(line)>18 && isequal(line(1:18),'MAX_CDNA_INSERTION')
  elseif length(line)>21 && isequal(line(1:21),'TERMINAL_EXON_END_TOL')
  elseif length(line)>14 && isequal(line(1:14),'MIN_INTRON_LEN')
    g_min_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_min_intron_len) & g_min_intron_len<1000) ;
  elseif length(line)>14 && isequal(line(1:14),'MAX_INTRON_LEN')
    g_max_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_max_intron_len) & g_max_intron_len>1000) ;
  elseif length(line)>18 && isequal(line(1:18),'MIN_EST_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_PROT_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_CDNA_COVER_FRAC')
  
  elseif length(line)>18 && isequal(line(1:18),'BLAT_BEST_HIT_ONLY')
  
  elseif length(line)>20 && isequal(line(1:20),'BLAT_BEST_HIT_MARGIN')

  elseif length(line)>8 && isequal(line(1:8),'ALPHABET')
    g_alphabet=deblank(line(10:end)) ;
  elseif length(line)>7 && isequal(line(1:7),'CONTIGS'),
    for i=1:str2num(line(8:end))
      l2=fgetl(fd) ; 
      idx=strfind(l2,'  ') ;
      while ~isempty(idx), 
        l2(idx)=[] ;
        idx=strfind(l2,'  ') ;
      end ;
      l2(l2==' ')=sprintf('\t') ;
      elems=separate(l2) ; 
      assert(length(elems)==3) ;
      g_contig_names{i}=elems{1} ;
      g_flat_fnames{i}=path_with_basedir(elems{2}, g_basedir) ;
      g_fasta_fnames{i}=path_with_basedir(elems{3}, g_basedir) ; 
    end ;
  elseif length(line)>8 && isequal(line(1:8),'ESTFILES'),
    for i=1:str2num(line(9:end))
      g_est_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'PROTFILES'),
    for i=1:str2num(line(10:end))
      g_prot_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'CDNAFILES'),
    for i=1:str2num(line(10:end))
      g_cdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>11 && isequal(line(1:11),'FLCDNAFILES'),
    for i=1:str2num(line(12:end))
      g_flcdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>15 && isequal(line(1:15),'ANNOTATIONFILES'),
    for i=1:str2num(line(16:end))
      g_annotation_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif ~isempty(line),
    error(sprintf('cannot understand "%s" in config file', line)) ;
  end ;
end ;
fclose(fd) ;

genome_info=[] ;
genome_info.est_fnames=g_est_fnames ;
genome_info.prot_fnames=g_prot_fnames ;
genome_info.cdna_fnames=g_cdna_fnames ;
genome_info.flcdna_fnames=g_flcdna_fnames ;
genome_info.contig_names=g_contig_names ;
genome_info.flat_fnames=g_flat_fnames ;
genome_info.fasta_fnames=g_fasta_fnames ;
genome_info.annotation_fnames=g_annotation_fnames ;
genome_info.alphabet=g_alphabet ;
genome_info.basedir=g_basedir ;
genome_info.max_intron_len = g_max_intron_len ;
genome_info.min_intron_len = g_min_intron_len ;
genome_info.merge_est_transcripts = g_merge_est_transcripts ;
g_genome_info=genome_info ;

return ;


function p=path_with_basedir(p1,basedir)
  
  if p1(1)=='/',
    p=p1; 
    return ;
  else
    if basedir(end)=='/',
      p=[basedir p1] ;
    else
      p=[basedir '/' p1] ;
    end ;
  end ;
  
  

return

function [str,valid_splice,ok] = load_genomic(contig,strand,start,stop, genome_info,check_splice)
% function [str,valid_splice,ok]=load_genomic(contig,strand,start,stop, genome_info,check_splice)

if nargin<6
  check_splice=0;
end ;
ok=1 ;

if ~check_splice 
  valid_splice=[];
  % fprintf(1,'no splice site consensus check!')
end ;


if length(start)>1 | length(stop)>1,
  if nargout<3
    assert(length(start)==length(stop)) ;
    assert(all([start(2:end)-stop(1:end-1)]>=0));
  else
    if ~(length(start)==length(stop)) || ~(all([start(2:end)-stop(1:end-1)]>=0)),
      ok=0 ;
      str='' ;
      valid_splice = [] ;
      return ;
    end ;
  end ;
  valid_splice = zeros(length(start),2); 
  if strand == '+', 
    idx = 1:length(start) ; 
  else 
    idx = length(start):-1:1 ; 
  end ;
  str = '' ;
  for i=idx  %1:length(start)
    seq = load_genomic(contig, strand, start(i)-5, stop(i)+5, genome_info) ;
    str = [str seq(6:end-5)] ;

    % check internal splicing sites
    if check_splice
      % if (i~=1 && strand == '+') || (i~=length(start) && strand == '-')
      if (strand == '+') || (i~=length(start) && strand == '-')
        valid_splice(i,1) = isequal(seq(4:5),'ag') ;
        %if ~isequal(seq(4:5),'ag'), fprintf('acc: %s %c\n', seq(3:6), strand); end ;
        % assert(isequal(seq(4:5),'ag'))
      end ;
      %if i~=idx(end)
      % if (i~=1 && strand == '-') || (i~=length(start) && strand == '+')
      if (strand == '-') || (i~=length(start) && strand == '+')        
        valid_splice(i,2) = ismember(seq(end-5:end-4), {'gt','gc'}) ;
        %if ~isequal(seq(end-4:end-3),'gt')||isequal(seq(end-4:end-3),'gc'), fprintf('don: %s %c\n', seq(end-5:end-2), strand); end ;
        % assert(isequal(seq(end-4:end-3),'gt')||isequal(seq(end-4:end-3),'gc'))
      end ; 
    end ;
  end
  return ;
else
  valid_splice=[0 0];
end ;


if ischar(contig),
  %contig_idx = find(ismember(upper(genome_info.contig_names),upper(contig))) ;
  contig_idx = strmatch(upper(contig), upper(genome_info.contig_names), 'exact') ;
  assert(all(size(contig_idx)==1) & ~isempty(contig_idx)) ;
else
  contig_idx = contig ;
end ;
  
fname=genome_info.flat_fnames{contig_idx} ;
if genome_info.alphabet(1)=='a',
  NN='n' ;
else
  NN='N' ;
end ;

d=dir(fname) ; left_n = 0 ; right_n=0 ;
if isinf(stop), stop=d.bytes; end ;

if start<1 | stop>d.bytes,
  warning('load_genomic:contig_boundary', 'boundary of contig reached (start: %i, stop: %i, d.bytes: %i), padding with "n"', start, stop,d.bytes) ;

  if start<1,
    left_n = -(start-1) ;
    start = 1 ;
  end ;
  if stop>d.bytes,
    right_n = stop-d.bytes ;
  end ;
  if start>d.bytes && stop>d.bytes,
    str = char(NN*ones(1,stop-start)) ;
    return ;
  end ;
end ;

fd=fopen(fname,'r') ;
fseek(fd, start-1, -1) ;
str=char(fread(fd,stop-start+1, 'char=>char'))' ;
fclose(fd) ;
if left_n>0,
  str=char([NN*ones(1,left_n) str]) ;
end ;
if right_n>0,
  str=char([str NN*ones(1,right_n)]) ;
end ;

assert(length(str)==stop-start+left_n+1)
%let = bigunique(str) ;
let = unique(str) ;
if genome_info.alphabet(1)=='a' && ~isempty(setdiff(let,'acgtn')),
  % warning(sprintf('invalid letter in genome! ("%s")', setdiff(let,'acgtn')));
  let = setdiff(let, 'acgtn') ;
  for c=let,
    str(str==c) = 'n';
  end ;
elseif genome_info.alphabet(1)=='A' && ~isempty(setdiff(let,'ACGTN')),
  warning(sprintf('invalid letter in genome! ("%s")', setdiff(let,'ACGTN')));
  let = setdiff(let, 'ACGTN') ;
  for c=let,
    str(str==c) = 'N';
  end ;
end

if strand=='-',
  str=reverse_complement(str) ;
end ;

return ;

return

function x = fexist(filename,gz)
% FEXIST Check if file exists
%        FEXIST('filename') returns:
%        0 if filename does not exist
%        1 if filename exists in the given path

if nargin<2,
  gz=0;
end ;

fd = fopen(filename);
if fd ~= -1
  x = 1;
  fclose(fd);
elseif gz,
  fd = fopen([filename '.gz']);
  if fd~=-1,
    x=1 ;
    fclose(fd) ;
  else
    x=0 ;
  end ;
else
  x = 0;
end

return

function f=separate(str, delim, merge_multiple) ;
% f=separate(str, delimiter, merge_multiple_delimiters) ;
  
if nargin<2, delim=sprintf('\t') ; end ;
if nargin<3, merge_multiple=0 ; end ;
f={} ;
idx=[0 find(str==delim) length(str)+1] ;
if merge_multiple,
  idx2=find(idx(1:end-1)~=idx(2:end)-1) ;
  idx=[idx(idx2) idx(end)] ;
end 
for i=1:length(idx)-1
  f{i}=deblank(str(idx(i)+1:idx(i+1)-1)) ;
end ;

return

function genes = init_genes(num_genes,PAR,genes)
% genes = init_genes(num_genes,PAR,genes)
  
if nargin<1
  num_genes=0;
end
if nargin<2 || isempty(PAR),
  PAR.organism.name =[];
end
    
genes.id = [];
genes.anno_id = [];
genes.confgenes_id = [];
genes.name = '' ;
genes.alias = [];
genes.name2 = []; 
genes.strand = '';
genes.chr = '';
genes.chr_num = [];
genes.paralogs = [];

genes.start = [];
genes.stop = [];

genes.transcripts = {};
genes.transcript_info = [];
genes.transcript_status = [];
genes.transcript_valid = [];
genes.exons = {}; 
genes.exons_confirmed = {};
genes.cds_exons = {}; 
genes.utr5_exons = {}; 
genes.utr3_exons = {}; 
genes.tis = {};
genes.tis_conf = [];
genes.tis_info = [];

genes.cdsStop = {};
genes.cdsStop_conf = [];
genes.cdsStop_info = [];

if isequal(PAR.organism.name,'C_elegans')||(isfield(PAR.organism,'clade')&&isequal(PAR.organism.clade,'nematode'))
  genes.transacc = {};
  genes.transacc_info = [];
  genes.transacc_conf = [];
end
genes.tss = {};
genes.tss_info = [];
genes.tss_conf = [];
genes.cleave = {};
genes.cleave_info = [];
genes.cleave_conf = [];
genes.polya = {};
genes.polya_info = [];
genes.polya_conf = [];

genes.is_alt = [];
genes.is_alt_spliced = [];
genes.is_valid = [];
genes.transcript_complete = [];
genes.is_complete = [];
genes.is_correctly_gff3_referenced = [];

genes.splicegraph =[];

if isequal(PAR.organism.name,'C_elegans')||(isfield(PAR.organism,'clade')&&isequal(PAR.organism.clade,'nematode'))
  genes.in_operon=[];
end

if num_genes==0
  genes=genes([]);
elseif num_genes>1,
  genes(num_genes) = genes ;
end

return

function [attributes, type_name, elems, action] = parse_starter(fd, types, types_IDs, contig, source)
% parse_starter(fd)

attributes = {} ;
type_name = '' ;
elems = {} ;
action = '' ;


Line = fgetl(fd);
elems = separate(Line);
assert(~isempty(elems{1}));

if Line(1)=='#',
  action = 'continue' ;
  return ;
end
if ~isempty(contig)
  if ~isequal(upper(elems{1}),upper(contig)),
    action='continue' ;
    return ;
  end
end
if ~ischar(Line),
  action='break' ;
  return ;
end

if isempty(elems{4}) | isempty(elems{4})
  action = 'continue' ;
  return ;
end


assert(str2double(elems{4})<=str2double(elems{5}) || str2double(elems{5})<0)

if~(elems{7}=='+' || elems{7}=='-') ;
  if elems{7}~='.',
    warning('strand missing')
  end ;
end

if ~isempty(source)
  if ~isequal(elems{2},source)
    error('something wrong with source')
  end  
end

if ~ismember(elems{3},types) && ~ismember(elems{3},types_IDs)
  action = 'continue' ;
  return ;
end
attributes = separate(elems{9}, ';');

if ismember(elems{3},types);
  type_name = elems{3};
elseif ismember(elems{3},types_IDs);
  type_name =types(isequal(elems{3},types_IDs));
else
  error('wrong type')
end

action='' ;

return

function PAR=create_PAR(Exp, PAR_config, verb)
%  PAR=create_PAR(Exp,PAR_config,verb)
%
% creates PAR with all parameters for one genefinding
% experiment. For a new Organism X a new template file must be added to /template_files/Organisms/X.m. 
% (e.g. C_elegans.m) 
% For each signal a standard method is applied (usually SVM with one or
% more spectrum kernels)
%  
% INPUT  - Exp: filename of current experiment  (e.g. exp_C_elegans_nGASP)
%        - verb: verbose
%
% OUTPUT - PAR. struct with all information needed for genefinding experiment  
  
Signals = {'acceptor','donor','tis','cdsStop','tss','cleave','polya','transacceptor'};
sig_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
Methods = {'SVM_WD','SVM_WD','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_WD'};

content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
Methodc = 'SVM_Cont';
changeable_fields = {'organism','label_source','Signals','Contents','LSL','RPROC','SETs','regions','tasks','Source_organisms','FN'} ;

if nargin<3,
  verb=0;
end


PAR = init_PAR;

if nargin>0
  fprintf('parameters from exp file: %s \n',Exp)
  if ~exist('PAR_config') || isempty(PAR_config),
    PAR_exp = feval(Exp) ;
  else
    try,
      PAR_exp = feval(Exp, PAR_config) ;
    catch
      PAR_exp = feval(Exp) ;
    end 
    clear PAR_config ;
  end ;
  assert(all(ismember(fieldnames(PAR_exp),changeable_fields)))
  if ~isfield(PAR_exp,'Contents')
    PAR_exp.Contents = struct;
  end
  if ~isfield(PAR_exp,'Signals')
    PAR_exp.Signals = struct;
  end
  PAR.organism = set_default(PAR_exp,'organism',PAR.organism)  ;  
  PAR.label_source = set_default(PAR_exp,'label_source',PAR.label_source) ;
  PAR.RPROC = set_default(PAR_exp,'RPROC',PAR.RPROC)  ; 
  PAR.SETs = set_default(PAR_exp,'SETs',PAR.SETs)  ; 
  PAR.SETs = make_partitions(PAR.SETs);
  PAR.regions = set_default(PAR_exp,'regions',PAR.regions)  ; 
  PAR.tasks = set_default(PAR_exp,'tasks',PAR.tasks)  ; 
  PAR.Source_organisms = set_default(PAR_exp,'Source_organisms',PAR.Source_organisms)  ; 
end

if isempty(PAR.organism.name),
  [organism, label_source] = dummy_organism(PAR.label_source, sig_names);
else
  [organism, label_source] = feval(PAR.organism.name,PAR.label_source,sig_names);
end
PAR.organism = set_default(PAR,'organism',organism)  ;
PAR.label_source = set_default(PAR,'label_source',label_source)  ;

%-----------------------------
%%% prepare all signals
%-----------------------------

for i=1:length(Signals)
  Signal = feval(Signals{i},PAR.label_source);
  if exist('PAR_exp')
    if isfield(PAR_exp.Signals, sig_names{i}) && isfield(PAR_exp.Signals.(sig_names{i}),'method')
      me = PAR_exp.Signals.(sig_names{i}).method;
      PAR_exp.Signals.(sig_names{i}) = rmfield(PAR_exp.Signals.(sig_names{i}),'method');
    end
    Signal = set_default(PAR_exp.Signals,sig_names{i},Signal)  ;
  end
  Signal.method = feval(Methods{i},Signal);
  if exist('me')
    PAR_exp.Signals.(sig_names{i}).method = me;
    Signal = set_default(PAR_exp.Signals,sig_names{i},Signal)  ;
    clear me
  end
  % Signal = Signal.domain_adapt.transfer_fct(Signal);
  PAR.Signals.(sig_names{i}) = Signal;
end

%-----------------------------
%%% prepare all contents
%-----------------------------
for i=1:length(content_names)
  Content = feval(content_names{i});
  if exist('PAR_exp')
    if isfield(PAR_exp.Contents, content_names{i}) && isfield(PAR_exp.Contents.(content_names{i}),'method')
      me = PAR_exp.Contents.(content_names{i}).method;
      PAR_exp.Contents.(content_names{i}) = rmfield(PAR_exp.Contents.(content_names{i}),'method');
    end
    Content = set_default(PAR_exp.Contents,content_names{i},Content)  ;
  end
  Content.method = feval(Methodc,Content);
  if exist('me')
    PAR_exp.Contents.(content_names{i}).method = me;
    Content = set_default(PAR_exp.Contents,content_names{i},Content)  ;
    clear me
  end
  % Content = Content.domain_adapt.transfer_fct(Content);
  PAR.Contents.(content_names{i}) = Content;
end

%-----------------------------
%%% prepare LSL
%-----------------------------

LSL = lsl_template; 
% LSL = LSL.domain_adapt.transfer_fct(LSL);
if exist('PAR_exp')
  LSL = set_default(PAR_exp,'LSL',LSL)  ; 
end
PAR.LSL=LSL;

%-----------------------------
%%% prepare filenames
%-----------------------------
FN = init_FN;
if  exist('PAR_exp')
  FN = set_default(PAR_exp,'FN',FN)  ;
end

FN = create_filenames(FN,PAR.organism,PAR.Signals,PAR.Contents,verb);
if  exist('PAR_exp')
  FN = set_default(PAR_exp,'FN',FN)  ;
end
PAR.FN=FN;

%-----------------------------
%%% Add polya consensus if file available
%-----------------------------

if fexist(PAR.FN.input_sig.polya.fn_polya)
  load(PAR.FN.input_sig.polya.fn_polya,'consensus')
  PAR.Signals.polya.consensus = consensus;
end

%PAR.FN.source_directory = '~/svn/projects/genefinding/';
%[tmp,PAR.FN.svn_info]=unix('svn info');

PAR.FN.svn_info = '' ;

fprintf('loading model from %s\n', PAR.LSL.method.model_fct)
PAR.model = feval(PAR.LSL.method.model_fct,PAR.organism,PAR.LSL, ...
                  PAR.Signals);


return

function PAR=init_PAR(signal_names,content_names);
 
  
% -------------------------------------------------------------------------------------
% Experiments
% -------------------------------------------------------------------------------------
if nargin<1
  signal_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
end
if nargin<2
  content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
end




PAR.date = date;
PAR.user = whoami;
  
PAR.organism.clade = [];
PAR.organism.name = '';
PAR.organism.full_name = [];
PAR.organism.release = [];
PAR.organism.version = [];
PAR.organism.genebuilt = [];

% -------------------------------------------------------------------------------------
% info regarding original gene structure
% -------------------------------------------------------------------------------------


info_genes.transcript.anno = -1;
info_genes.transcript.est = 0;
info_genes.transcript.cDNA = 1;
info_genes.transcript.fulllength = 2;

info_genes.transcript_status.confirmed = 1;
info_genes.transcript_status.partially_confirmed = 0;
info_genes.transcript_status.predicted = -1;


info_genes.pseudo.dummy = [];

% -------------------------------------------------------------------------------------
% info regarding label source
% -------------------------------------------------------------------------------------

label_source.use_anno = [];
label_source.use_confirmed = [];
for j=1:length(signal_names)
  signal_name = signal_names{j};
  label_source.(signal_name).anno = [];
  label_source.(signal_name).est = [];
  label_source.(signal_name).cDNA = [];
  label_source.(signal_name).fulllength = [];
  label_source.(signal_name).db = [];
  label_source.(signal_name).from_fn_candsites = [];
  
  info_genes.(signal_name).anno = -1;
  info_genes.(signal_name).est = 0;
  info_genes.(signal_name).cDNA = 1;
  info_genes.(signal_name).fulllength = 2;
  info_genes.(signal_name).db = 3;
  info_genes.(signal_name).from_fn_candsites = 4;
end
for j=1:length(content_names)
  content_name = content_names{j};
  label_source.(content_name).from_fn_candsites = [];
end

label_source.tis.maxorf = [];
info_genes.tis.maxorf = 5;
label_source.cdsStop.maxorf = [];
info_genes.cdsStop.maxorf = 5;

label_source.transacc.anno_SL1 = [];
label_source.transacc.anno_SL2 = [];
label_source.transacc.pred = [];
info_genes.transacc.anno_SL1 = 5;
info_genes.transacc.anno_SL2 = 6;
info_genes.transacc.pred = 7;

label_source.polya.from_cleave = [];
info_genes.polya.from_cleave = 5;


PAR.label_source = label_source;
PAR.info_genes = info_genes;


PAR.Signal_name = [];
for j=1:length(signal_names)
  signal_name = signal_names{j};
  PAR.Signals.(signal_name) = struct;
end

cnt=0;
PAR.Content_name = [];
for j=1:length(content_names)
  PAR.Contents.(content_names{j}) = struct;
  PAR.Contents.(content_names{j}).name = content_names{j};
  PAR.Contents.(content_names{j}).type = cnt;
  cnt=cnt+1;
end

PAR.LSL.method = struct;

% -------------------------------------------------------------------------------------
% memory requirements
% -------------------------------------------------------------------------------------

PAR.RPROC.collect_jobs = 0 ; 
PAR.RPROC.options.ncpus = 1 ; 
PAR.RPROC.options.waitonfull = [];
PAR.RPROC.options.maxjobs = [] ;
PAR.RPROC.options.immediately = 0;
PAR.RPROC.options.immediately_bg = 0;
PAR.RPROC.options.addpaths = [];
PAR.RPROC.options.rmpaths = [];

PAR.RPROC.MEMREQ = [];
PAR.RPROC.time_req = [];
PAR.RPROC.exm_per_batch = 1 ;


% -------------------------------------------------------------------------------------
% SPLITS AND PARTITIONS
% -------------------------------------------------------------------------------------

PAR.SETs.num_splits = 5;
PAR.SETs.num_partitions = 5;
PAR.SETs.train_subsample_neg = 1;
PAR.SETs.test_subsample_neg = 1;
PAR.SETs.train_subsample_pos = 1;
PAR.SETs.test_subsample_pos = 1;
%for lsl
PAR.SETs.num_train = [] ; 
PAR.SETs.ignore_incomplete = 1 ; 


% -------------------------------------------------------------------------------------
% original data processing stuff
% -------------------------------------------------------------------------------------

PAR.regions.max_length = 1000000;
PAR.regions.min_genes_per_region = 1;
PAR.regions.offset_test = 20000;
PAR.regions.offset_train = 1000;
PAR.regions.minCutDist = 250;
PAR.regions.cluster_paralogs = 0;
PAR.regions.merge_confirmed = 0;

% -------------------------------------------------------------------------------------
% TASKS
% -------------------------------------------------------------------------------------

PAR.tasks.signals.prepare_labels = 0;
PAR.tasks.signals.train = 0;
PAR.tasks.signals.modelsel = 0;
PAR.tasks.signals.learn_plifs = 0;
PAR.tasks.signals.get_train_error = 0;

PAR.tasks.signals.pred_on_genome = 0; 
PAR.tasks.signals.verbose = 0; 

PAR.tasks.signals.run_locally = 0;

% -------------------------------------------------------------------------------------
% transfer from other organisms
% -------------------------------------------------------------------------------------

PAR.Source_organisms.names = {};
PAR.Source_organisms.dir_names = {};
PAR.Source_organisms.exp_names = {};


% -------------------------------------------------------------------------------------
% FN
% -------------------------------------------------------------------------------------
PAR.FN = struct;


return

function Signal = acceptor(label_source,signal_name)

% Signal = acceptor(label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field acc (transcc) is modified.  
%        - label_source is a struct that should contain a field
%          (signal_name). this is used in the called function splicesite_template.
%        - name can either be 'acc' or 'transacc'  
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template
% (function acceptor is also called from transacc)
  
  
if nargin<3 
  signal_name = 'acc';
end

Signal = splicesites_template(label_source,signal_name);

%%%% SET ACCEPTOR STUFF

Signal.consensus = {'AG'};
Signal.offset = 2;

Signal.lwin = {[-60]} ;
Signal.rwin = {[80]} ;

Signal.order = [ 22];
Signal.shift = 0.5;
Signal.C = [3]; 

Signal.Conf_names = {Signal.Conf_names{:},'alt_3prime'};

Signals.(signal_name) = Signal;



return

function Signal = donor(label_source)

% Signals = donor(Signals,signal_name,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field don is modified.
%        - label_source is a struct that should contain a field don. 
%          this is used in the called function signal_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template

 
signal_name = 'don';

Signal = splicesites_template(label_source,signal_name);

%%%% SET DONOR STUFF

Signal.consensus = {'GC' 'GT'};
Signal.offset = 0;

Signal.lwin = {[-80]};
Signal.rwin = {[60]};

Signal.order = [22];
Signal.shift = 0.5;
Signal.C = [3]; 

Signal.Conf_names = {Signal.Conf_names{:},'alt_5prime'};




return

function Signal = tss(label_source,has_transacc)
  
% Signals = tss(Signals,label_source,has_transacc)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field tss is modified.
%        - label_source is a struct that should contain a field tss. 
%          this is used in the called function signal_template.
%        - has_transacc: is per default set to zero, only 1 for nematodes. 
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template

  
if nargin<3
  has_transacc=0; 
end
   
signal_name = 'tss';

Signal = transcriptends_template(label_source,signal_name);

%%%% SET TSS STUFF


Signal.lwin_big = 601;
Signal.rwin_big = 901;

Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};

Signal.lwin = { [-600] [-70] [   0] };
Signal.rwin = { [+100] [+70] [+900] };
Signal.wordlen = { [4] [] [4] };

Signal.order ={[] [16] []};
Signal.shift ={[] [0] []};
Signal.shift_const ={[] [28] []};
Signal.C = 2; % log2(C)=1.0

if has_transacc
  Signal.Conf_names = {Signal.Conf_names{:},'has_trans'};
  Signal.filter_label.train.rm_trans = 1;
  Signal.filter_label.eval.rm_trans = 1;
  Signal.filter_label.test.rm_trans = 1;
else
  Signal.filter_label.train.rm_trans = 0;
  Signal.filter_label.eval.rm_trans = 0;
  Signal.filter_label.test.rm_trans = 0;
end

%%%%%% for human
%Signal.sampling.downsample_neg = 1/160; % <=1 (for no downsampling: 1)



return

function Signal = cleave(label_source)

% Signals = cleave(Signals,signal_name,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field cleave is modified.
%        - label_source is a struct that should contain a field cleave. 
%          this is used in the called function signal_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template

  
signal_name = 'cleave';

Signal = transcriptends_template(label_source,signal_name);

%%%% SET CLEAVE STUFF

Signal.lwin_big = 410;
Signal.rwin_big = 810;

Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};

Signal.lwin = {[-400], [-60], [0]};
Signal.rwin = {[100], [60], [800]};
Signal.wordlen = {[5], [], [3]}; 

Signal.order = {[],[18],[]};
Signal.shift = {[],[0.1],[]};
Signal.shift_const = {[],[0],[]};
Signal.C = 1; % log2(C)=0



return

function Signal = polya(label_source,fn_polya)
  
% Signals = polya(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field polya is modified.
%        - label_source is a struct that should contain a field polya. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: polya -> signal_template

signal_name = 'polya' ; 
Signal = signal_template(label_source,signal_name);

%%%% SET POLYA STUFF

Signal.name = signal_name;
Signal.type = signal_name; 
Signal.label_fct = 'get_cand_others';
Signal.filter_fct = 'filter_cands';

Signal.offset = 0;

Signal.lwin_big = 310;
Signal.rwin_big = 310;

Signal.kernel_name= {'WDS','SP','SP','SP','SP','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.subkernel_weights = [1 0.2 0.2 0.2 0.2 0.2];

Signal.lwin = {-300 -300 -149 1 151 -300} ;
Signal.rwin = {306 -150 0 150 306 306};
Signal.shift={[0 0.1 0.4],[],[],[],[],[]};
%Signal.shift={[0.1],[],[],[],[],[]};
Signal.shift_const={[0 10 20],[],[],[],[],[]};
%Signal.shift_const={[0],[],[],[],[],[]};
Signal.wordlen={[],[4],[4],[4],[4],[4]};
Signal.order={[18],[],[],[],[],[]};
Signal.C = [0.01 0.5 10];

Signal.consensus = {''};
if exist('fn_polya')
  Signal.consensus_fn = fn_polya;
  if fexist(Signal.consensus_fn)
    load(Signal.consensus_fn,'consensus')
    Signal.consensus = consensus;
  end
end

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
%Signal.filter_label.eval.use_intergenic=0;
%Signal.filter_label.eval.use_alt=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;

Signal.filter_label.train.conf_pos.anno = 0;
Signal.filter_label.train.conf_pos.from_cleave = [201 301 401];
Signal.filter_label.train.conf_pos.db = 0;
Signal.filter_label.train.conf_pos.fulllength = 0;
Signal.filter_label.train.onlywithpos = 0; 

%Signal.filter_label.eval.conf_pos.anno = 0;
%Signal.filter_label.eval.conf_pos.from_cleave = [201 301 401];
%Signal.filter_label.eval.conf_pos.db = 0;
%Signal.filter_label.eval.conf_pos.fulllength = 0;
%Signal.filter_label.eval.onlywithpos = 0; 

Signal.filter_label.test.conf_pos.anno = 0;
Signal.filter_label.test.conf_pos.from_cleave = [201 301 401];
Signal.filter_label.test.conf_pos.db = 0;
Signal.filter_label.test.conf_pos.fulllength = 0;
Signal.filter_label.test.onlywithpos = 0; 


Signal.Conf_names = {Signal.Conf_names{:},['has_' signal_name]};



Signal.label_source_par.polya.wl = 1;
Signal.label_source_par.polya.POSconstr =1;
Signal.label_source_par.polya.maxpcCov = 95;
Signal.label_source_par.polya.logdir = [];

return

function Signal = tis(label_source)
  
% Signals = tis(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field tis is modified.
%        - label_source is a struct that should contain a field tis. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template

    
signal_name = 'tis';

Signal = translatends_template(label_source,signal_name);

%%%% SET TIS STUFF

Signal.consensus = {'ATG'};


Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.lwin = {[-200] [-30] [0]} ; 
Signal.rwin = {[0] [110] [180]}; 
Signal.shift={[],[0],[]};
Signal.shift_const={[],[0],[]};
Signal.order={[],[24],[]};  
Signal.wordlen = {[3] [] [5]}; 
Signal.C = 1; % log2(C)=1.0










return

function Signal = cdsStop(label_source)
  
% Signals = cdsStop(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field cdsStop is modified.
%        - label_source is a struct that should contain a field cdsStop. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template
   
signal_name = 'cdsStop';

Signal = translatends_template(label_source,signal_name);

%%%% SET cdsStop STUFF

Signal.consensus = {'TAA','TAG','TGA'};


Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.lwin = {[-200] [-110] [0]} ; 
Signal.rwin = {[0] [10] [180]};
Signal.shift={[],[0],[]};
Signal.shift_const={[],[0],[]};
Signal.order={[],[20],[]};  
Signal.wordlen = {[2] [] [4]}; 
Signal.C = 2.8284; % logC=1.5



return

function Signal = transacceptor(label_source)

% Signals = transacceptor(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field transcc is modified.  
%        - label_source is a struct that should contain a field
%          transacc. this is used in the called function splicesite_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tansacceptor -> acceptor -> splicesite_template -> signal_template

INF = 1e6;  
signal_name = 'transacc';

Signal = acceptor(label_source,signal_name);

%%%% SET transacc STUFF

Signal.lwin = {[-190]} ;
Signal.rwin = {[80]};


Signal.filter_label.train.conf.anno_SL1 = 0;
Signal.filter_label.train.conf.anno_SL2 = [0 INF];
Signal.filter_label.train.conf.pred = [0 INF];

Signal.filter_label.eval.conf.anno_SL1 = 0;
Signal.filter_label.eval.conf.anno_SL2 = [0 INF];
Signal.filter_label.eval.conf.pred = [0 INF];


Signal.filter_label.test.conf.anno_SL1 = 0;
Signal.filter_label.test.conf.anno_SL2 = [0 INF];
Signal.filter_label.test.conf.pred = [0 INF];


Signal.filter_label.train.use_intergenic = 1;
Signal.filter_label.train.use_alt = 0;
Signal.filter_label.eval.use_intergenic = 1;
Signal.filter_label.eval.use_alt = 0;
Signal.filter_label.test.use_intergenic = 1;
Signal.filter_label.test.use_alt = 0;
 
Signal.filter_label.train.use_pred = 1;
Signal.filter_label.eval.use_pred = 1;
Signal.filter_label.test.use_pred = 1;

Signal.filter_label.train.only_acc = 1;
Signal.filter_label.eval.only_acc = 1;
Signal.filter_label.test.only_acc = 1;

Signal.filter_label.train.pos_dist = 20;
Signal.filter_label.eval.pos_dist = 20;
Signal.filter_label.test.pos_dist = 20;

Signal.export_settings.conf_cum_thresh = 0.05 ;
Signal.export_settings.resolution = 1 ;

Signal.Conf_names = {Signal.Conf_names{:},'isacc','has_tss','has_trans','dist'};


Signal.label_source_par.transacc.win_anno_SL1 = 50;
Signal.label_source_par.transacc.win_anno_SL2 = 50;
Signal.label_source_par.transacc.win_pred = 5;
Signal.label_source_par.transacc.threshold_pred = 0.5;


return

function Signal = signal_template(label_source,signal_name)

% Signals = signal_template(Signal,label_source,signal_name)
% Signals is a struct with a field for each Signal. In this function only
% the field signal_name is added. Default values are set.


if nargin<2
  signal_name = 'dummy';
end

%%%%%

Signal.type = []; 
Signal.name = signal_name;
Signal.consensus = {[]};
Signal.offset = [];

Signal.lwin_big = [];
Signal.rwin_big = [];

Signal.label_fct = [];
Signal.filter_fct = 'filter_cands';
Signal.load_fct = 'load_data';
Signal.score_fct = [];

Signal.Conf_names = { 'altgenic','gene_id' };

Signal.sampling.downsample_neg = 1; % <=1 (for no downsampling: 1)  
Signal.sampling.mindist = 1; % >=1 (for all: 1)  
Signal.sampling.pos_neg_ratio = 1/10;

Signal.resolution   = 1; % for lsl

conf.USE_ALL = 0;
conf.conf_pos = struct;
conf.conf_pos_skip = struct;
conf.any_conf_pos = [];
conf.any_conf_pos_skip = [];
conf.conf_neg = struct;
conf.conf_neg_skip = struct;
conf.any_conf_neg = [];
conf.any_conf_neg_skip = [];
conf.use_intergenic=[];
conf.use_alt = [];
conf.use_label = 'label';

Signal.filter_label.train = conf ; 
Signal.filter_label.test = conf ;
Signal.filter_label.eval = conf ;

Signal.export_settings.conf_cum_thresh = 0 ;
Signal.export_settings.resolution = 1 ;

Signal.domain_adapt.name = 'target_only';
Signal.domain_adapt.Source_organisms = [];

Signal.method = struct;

label_source = label_source.(signal_name);
sources = fieldnames(label_source);
for i=1:length(sources)
  if label_source.(sources{i})
    Signal.Conf_names = {Signal.Conf_names{:},[sources{i} '_conf']};
    if isequal(signal_name,'acc')|isequal(signal_name,'don')
      Signal.Conf_names = {Signal.Conf_names{:},[sources{i} '_conf_skip']};
    end
  end
end




return

function Signal = splicesites_template(label_source,signal_name)

% Signals = splicesites_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name: 'don', 'acc','transacc'
%  
% OUTPUT Signals with modified field signal_name; 
%  
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template
%
% see also transcriptends_template/translat_template
  
  
INF=1e6;

Signal = signal_template(label_source,signal_name);

%%%% SET SPLICE SITE STUFF

Signal.type = 'splicesites'; 
Signal.label_fct = 'get_cand_splicesites';

Signal.lwin_big = 199;
Signal.rwin_big = 199;

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
Signal.filter_label.train.use_alt_neg=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;
Signal.filter_label.test.use_alt_neg=0;

Signal.filter_label.train.use_label = 'label';
Signal.filter_label.test.use_label = 'label';


Signal.filter_label.train.use_decoys = 1;
Signal.filter_label.test.use_decoys = 1;


Signal.filter_label.train.conf_pos.est_clusters = [0 INF];
Signal.filter_label.train.conf_pos.cDNA = [0 INF];
Signal.filter_label.train.conf_pos.fulllength = [0 INF];
Signal.filter_label.train.conf_pos.anno = [0:1] ;

Signal.filter_label.train.conf_pos_skip.est_clusters = [0 INF];
Signal.filter_label.train.conf_pos_skip.cDNA = [0 INF];
Signal.filter_label.train.conf_pos_skip.fulllength = [0 INF];
Signal.filter_label.train.conf_pos_skip.anno = [-inf] ;

Signal.filter_label.train.conf_neg.est_clusters = [0 INF];
Signal.filter_label.train.conf_neg.cDNA = [0 INF];
Signal.filter_label.train.conf_neg.fulllength = [0 INF];
Signal.filter_label.train.conf_neg.anno = [-1:1] ;

Signal.filter_label.train.conf_neg_skip.est_clusters = [0 INF];
Signal.filter_label.train.conf_neg_skip.cDNA = [0 INF];
Signal.filter_label.train.conf_neg_skip.fulllength = [0 INF];
Signal.filter_label.train.conf_neg_skip.anno = [-1:1] ;

Signal.filter_label.test = Signal.filter_label.train;
Signal.filter_label.eval = Signal.filter_label.train;

% Signal.save_confs = 1;
% Signal.save_alts = 1;


Signal.export_settings.conf_cum_thresh = 0.5 ;
Signal.export_settings.resolution = 1 ;


Signal.Conf_names = {Signal.Conf_names{:},'alt_exon','alt_intron'};





return

function Signal = transcriptends_template(label_source,signal_name)

% Signals = transcriptends_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name:'tss', 'cleave'
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template
%
% see also splicesite_template/translat_template
  
  
  
Signal = signal_template(label_source,signal_name);

%%%% SET TRANSCRIPT END STUFF

Signal.type = 'transcriptends'; 
Signal.label_fct = 'get_cand_others';


Signal.consensus = [];
Signal.offset = 0;

Signal.sampling.downsample_neg = 1/10; %% take randomly every 10th porsitions 
Signal.sampling.mindist = 10 ;


Signal.resolution   = 20; % for lsl

Signal.filter_label.train.conf_pos.anno = [0 1];
Signal.filter_label.train.conf_pos.est_clusters = [2:3];
Signal.filter_label.train.conf_pos.db = 0;
Signal.filter_label.train.conf_pos.fulllength = 0;
Signal.filter_label.train.onlywithpos=0; 

Signal.filter_label.train.use_intergenic=1;
Signal.filter_label.train.use_alt=0;

Signal.filter_label.test = Signal.filter_label.train;
Signal.filter_label.eval = Signal.filter_label.train;

Signal.export_settings.conf_cum_thresh = 0.01 ;
Signal.export_settings.resolution = 10 ;

Signal.Conf_names = {Signal.Conf_names{:},['has_' signal_name]};



return

function Signal = translatends_template(label_source,signal_name)

% Signals = translatends_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name:'tis', 'cdsStop'
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template
%
% see also splicesite_template/transcriptends_template
  
INF=1e6;
  
Signal = signal_template(label_source,signal_name);

%%%% SET TRANSLAT END STUFF

Signal.type = 'translatends' ;
Signal.label_fct = 'get_cand_others';

Signal.offset = 0;
Signal.lwin_big = 210;
Signal.rwin_big = 200;



Signal.filter_label.train.conf_pos.anno = [0 1] ;
Signal.filter_label.train.conf_pos.maxorf = INF;
Signal.filter_label.test.conf_pos.anno = [0 1] ;
Signal.filter_label.test.conf_pos.maxorf =INF;

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;

Signal.filter_label.eval = Signal.filter_label.test;

% PAR.label_source.cds.par.max_nof_paths = 100;
% PAR.label_source.cds.par.nof_best_kept = 2;


return

function method = method_template

% Signals = method_template(Signal,signal_name)
% Signals is a struct with a field for each Signal. In this function only
% the field Signal.(signal_name).method is added. Default values are set.

method.name = [];
method.train_fct = [];
method.predict_fct = [];

% PAR.method.svm_name = 'GPBTSVM';PAR.method.qpsize=500;
% use conservation information
% method.use_cons = 0;
% load training examples only once and calculate all models in one job
% method.submit_batch=0;

method.plif.bins = 50;
method.plif.min_incr = 1e-6;

method.use_cons = 0;
method.feature_type = [];
%assert(max(abs([PAR.Signals.(PAR.Signal_name).lwin{:}]))<PAR.Signals.(PAR.Signal_name).lwin_big)

%assert(max(abs([PAR.Signals.(PAR.Signal_name).lwin{:}]))<PAR.Signals.(PAR.Signal_name).rwin_big)




return

function method = SVM_WDS(Signal)

% Signals = SVM_WDS(Signals,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field (signal_name) is modified.
%          Particuarly a field method is added.
%        - signal_name can be something in {'acc','don','tss','cleave','polya','tis','cdsStop'};
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: SVM_WDS -> SVM -> method_template

  
method = SVM(Signal) ;
method.feature_type = 'char';


kernel.name = 'WDS';
kernel.order = Signal.order;
kernel.shift = Signal.shift;
kernel.lwin =  Signal.lwin{1};
kernel.rwin = Signal.rwin{1};

method.kernels{1} = kernel; 

return

function method = SVM_COMBINED(Signal)

% Signals = SVM_COMBINED(Signals,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field (signal_name) is modified.
%          Particuarly a field method is added.
%        - signal_name can be something in {'acc','don','tss','cleave','polya','tis','cdsStop'};
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: SVM_COMBINMED -> SVM -> method_template


method = SVM(Signal) ;

nofKernels = length(Signal.kernel_name);

assert(length(Signal.lwin)==nofKernels)
assert(length(Signal.rwin)==nofKernels)


for k=1:nofKernels
  for n = Signal.kernel_par_names 
    method.kernels{k}.(n{1}) = Signal.(n{1}){k};
  end
end
if isfield(Signal,'subkernel_weights')
  method.subkernel_weights = Signal.subkernel_weights;
end


return

function Content = intergenic

name = 'intergenic';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content=content_template(name)

Content.name = name;
Content.consensus = [];
Content.lwin_big = 0;
Content.rwin_big = 0;
%Content.order = [3 4 5];
Content.label_fct = 'get_cand_contents';
Content.load_fct = 'load_content_data';

Content.filter_label.train.USE_ALL = 1;
Content.filter_label.test.USE_ALL = 1;
Content.filter_label.eval.USE_ALL = 1;
Content.Conf_names = {} ;
Content.C = [1]; 

Content.wordlen = { };
Content.stepping = 1;
Content.offset = 0;

Content.export_settings.conf_cum_thresh = 0.5 ;
Content.export_settings.resolution = 1 ;


%PAR.Contents.type_descrs = { 'intergenic', 'utr_exon', 'cds_exon','intron', 'intercistronic' };
%PAR.Contents.type_abbrs = { 'ige' 'utr' 'cex' 'ino' 'ics' };
%PAR.Contents.num_types = length(PAR.Contents.type_descrs); 



return

function Content=utr5exon

name = 'utr5exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5],[6]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content = cds_exon

name = 'cds_exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content = frame0(info)

name = 'frame0';

Content = content_template(name);
Content.wordlen = {[3],[4],[5],[6]};
Content.stepping = 3;

Content.offset = mod(3-mod(Content.wordlen{1},3),3);
 

return

function Content = utr3exon

name = 'utr3exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content=intron

name = 'intron';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function method = SVM_Cont(Content)

  
method = SVM(Content);
method.feature_type = 'char';
method.center = [];
method.kernels{1}.name = 'LINEAR';
method.genomewide_predict_fct = 'svm_content_predict';

for k=1:length(Content.wordlen)
  method.kernels{k}.wordlen = Content.wordlen{k};
  method.kernels{k}.stepping = Content.stepping ;
  method.kernels{k}.offset = Content.offset ;
end



return

function LSL = lsl_template
  

method.name = 'lsl' ;
method.model_fct = 'make_model';
method.train_fct = 'lsl_train';
method.predict_fct = 'lsl_predict';

method.dt = datestr(now,'dd-mm-yyyy');
method.user = 'GS';
method.plif_bins   = 20 ;   % number of plif bins

%%%

%%Signals
use.signals.trans = 0 ;
use.signals.transacc = 0 ;
use.signals.tss = 1;
use.signals.polya = 1;
use.signals.cleave =  1;

%%States
use.states.SL1 = 0 ;
use.states.SL2 = 0;

%%transitions
use.transitions.allow_missing_tss = 0;
use.transitions.one_gene = 0 ; % force exactly one gene
use.transitions.use_multi = 1 ; % allow zero or more than one gene 

%%loss
use.loss.orf_only = 0;
use.model.one_gene = 0;
use.model.allow_missing_tss = 0;
use.model.use_multi = 1;

%%Segments & lengths
use.segments.transexon = 0 ;
use.segments.intercistronic = 0 ;
use.segments.intergenictrans = 0;
use.segments.polya_tail = 1;

%%Content 

use.contents.frame = 1;
use.contents.pre_comp = 1;

%%additional features
use.addfeatures.conservation = 0 ;
use.addfeatures.ests = 0 ;
use.addfeatures.proteins = 0 ;
use.addfeatures.tiling = 0 ;
use.addfeatures.rna_seq = 0 ;

% keyboard
method.use = use;



%----- REGULARIZATION
%C = 1 ;
C = 1e-1;
%%% linear regularization
method.par_ms.C_regul.lengths = C ;        % linear smoothness for len_hist
method.par_ms.C_regul.signals = C ;        % for sig_hist
method.par_ms.C_regul.contents = C ;       % for content_hist
method.par_ms.C_regul.transitions = C ;    % for

method.par_ms.C_regul.tiling = C ;  
method.par_ms.C_regul.rna_seq = C ;  


%%% quadratic regularization
C_sq = 1e-1;

method.par_ms.C_regul.transitions_sq = C_sq;
method.par_ms.C_regul.plif_ys_sq = C_sq*0.1 ;  % small_plif_ys_penalty
method.par_ms.C_regul.smoothness_sq = C_sq ;


%----------- for predictions only
method.par_ms.cleave_offset = 6;
%-----------

method.solver = 'dual';
method.ignore_init_file = 1;

method.opt_step = 1000 ;
method.max_neg = 2 ;
method.margin = 1 ;

method.INF = 1e20;
method.max_num_iterations = 100;

method.exm_per_solve = 400 ;
% method.exm_per_batch = 5 ;

%method.cleave_offset = 6 ;
% -------
% -------
LSL.method = method;


return

function lengths_range = default_lengths

lengths_range.intergenic = [0, 21000] ;
%lengths_range.intergenictrans = [0, 21000] ;
lengths_range.utr5exon   = [0, 8000] ;
% lengths_range.transexon  = [1, 1000] ;
lengths_range.single_cds_exon = [100, 8000] ; %%%100, from don => acc+stop
lengths_range.first_cds_exon  = [3, 8000] ;
lengths_range.middle_cds_exon   = [3, 8000] ;
lengths_range.last_cds_exon   = [1, 8000] ;
lengths_range.utr3exon    = [1, 8000] ;
lengths_range.polya_tail  = [0, 250] ;
% lengths_range.intercistronic = [1, 5000] ;
lengths_range.intergenic_long = [10500-50, 10500+50] ; % due to irregular spacing 
lengths_range.intron      = [30, 21000] ;%%%30, from don => acc+stop

return

function user_name = whoami
% user_name = whoami
  
global whoami_user_name  
  
if isempty(whoami_user_name)
  
  [ret, whoami_user_name]=unix('whoami') ; 
  if ret~=0,
    if keyboard_allowed(),
      keyboard ;
    end ;
    whoami_user_name='unknown' ;
  end ;
end ;

user_name= whoami_user_name ;
idx=find(user_name<30, 1, 'first') ;
user_name = user_name(1:idx-1) ;

return

function PAR=exp_galaxy(PAR_config)
% PAR=exp_galaxy(PAR_config)

PAR.SETs.num_partitions=5 ;
PAR.SETs.num_splits=5 ;

PAR.FN.input_dir=PAR_config.dir ;
PAR.FN.output_dir=PAR_config.dir;
PAR.FN.genome_dir=PAR_config.dir ;
PAR.FN.input.fn_genome_config = PAR_config.genome_config ;

% preparing RPROC settings
PAR.RPROC.MEMREQ = 6000 ;
PAR.RPROC.collect_jobs = 0 ;
PAR.RPROC.exm_per_batch = 1 ;
if 0
PAR.RPROC.options.addpaths = {'~/svn/projects/genefinding/sensors',
                    '~/svn/projects/genefinding/data_processing_sensors',
                    '~/svn/projects/genefinding/contents',
                    '~/svn/projects/genefinding/galaxy/tools/',
                    '~/svn/projects/genefinding/utils',
                    '~/svn/projects/genefinding/shogun',
                    '~/svn/projects/genefinding/data_processing_lsl',
                    '~/svn/tools/genomes',
                    '~/svn/tools/utils'} ;

end

PAR.RPROC.options.addpaths = {} ;

signals = {'acc','don','tis','cdsStop','tss','cleave'};
for i=1:length(signals)
  PAR.label_source.(signals{i}).from_fn_candsites = 1 ;
  PAR.Signals.(signals{i}).filter_label.train.USE_ALL = 1;
  PAR.Signals.(signals{i}).filter_label.test.USE_ALL = 1;
  PAR.Signals.(signals{i}).filter_label.eval.USE_ALL = 1;
  PAR.Signals.(signals{i}).Conf_names = {} ;
  PAR.Signals.(signals{i}).method.threads = 1;
end


if 0,
  PAR.Signals.acc.order = 10 ;
  PAR.Signals.acc.shift = 0 ;

  PAR.Signals.don.order = 10 ;
  PAR.Signals.don.shift = 0 ;
  
  PAR.Signals.tis.order =  {[],[5],[]};
  PAR.Signals.tis.shift =  {[],[0],[]};
  
  PAR.Signals.cdsStop.order = {[],[5],[]};
  PAR.Signals.cdsStop.shift = {[],[0],[]};
  
  PAR.Signals.tss.order = {[],[5],[]};
  PAR.Signals.tss.shift = {[],[0],[]};
  
  PAR.Signals.cleave.order = {[],[5],[]};
  PAR.Signals.cleave.shift = {[],[0],[]};

end


contents = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};

for i=1:length(contents)
  PAR.label_source.(contents{i}).from_fn_candsites = 1 ;
  PAR.Contents.(contents{i}).filter_label.train.USE_ALL = 1;
  PAR.Contents.(contents{i}).filter_label.test.USE_ALL = 1;
  PAR.Contents.(contents{i}).filter_label.eval.USE_ALL = 1;
  PAR.Contents.(contents{i}).Conf_names = {} ;
  PAR.Contents.(contents{i}).method.threads = 1;
end

PAR.LSL.method.use.signals.polya = 0 ;
PAR.LSL.method.exm_per_solve = 100 ;

PAR.regions.max_length = 30000 ;


if 1,

%----- REGULARIZATION
method=PAR.LSL.method ;

%C = 1 ;
C = 1e-3;
%%% linear regularization
method.par_ms.C_regul.lengths = C ;        % linear smoothness for len_hist
method.par_ms.C_regul.signals = C ;        % for sig_hist
method.par_ms.C_regul.contents = C ;       % for content_hist
method.par_ms.C_regul.transitions = C ;    % for

method.par_ms.C_regul.tiling = C ;  
method.par_ms.C_regul.rna_seq = C ;  


%%% quadratic regularization
C_sq = 1e-3;

method.par_ms.C_regul.transitions_sq = C_sq;
method.par_ms.C_regul.plif_ys_sq = C_sq*0.1 ;  % small_plif_ys_penalty
method.par_ms.C_regul.smoothness_sq = C_sq ;

PAR.LSL.method=method ;

end

return

function new_field = set_default(old_struct,field,defaults,silent)  ;

% new_struct = set_default(old_struct,defaults,overwrite)  ;
% sets all values of all fields in old_struct to the default values specified in
% defaults IF field is empty in old_struct. If overwrite is set all entries
% will be overwritten.
% defaults must be a struct.
%
% e.g. 
% default.use_cons = 0;
% default.plif.bins = 50;
% method.use_cons = 0;
% method = set_default(method,default,1) 

if ~isfield(old_struct,field)
  new_field = defaults;
  return
end  

old_struct = old_struct.(field);

if nargin<4
  silent = 1 ;
end

names = fieldnames(defaults);
for j=1:length(names)
  if ~isfield(old_struct,names{j}) || isempty(old_struct.(names{j}))||...
        (iscell(old_struct.(names{j}))&&length(old_struct.(names{j}))==1&&isempty(old_struct.(names{j}){1}))
    if ~isfield(old_struct,names{j}) && ~silent
      warning('new field added')
      names{j}
    end
    
    old_struct.(names{j}) = defaults.(names{j});
  elseif isstruct(defaults.(names{j}))
    old_struct.(names{j}) = set_default(old_struct,names{j},defaults.(names{j}));
    %names2 =  fieldnames(defaults.(names{j}));
    %for k= 1:length(names2)
    %  if ~isfield(old_struct.(names{j}),names2{k})||isempty(old_struct.(names{j}).(names2{k}))||overwrite
    %    old_struct.(names{j}).(names2{k}) = defaults.(names{j}).(names2{k});
    %  end
    %end
  end
end
% if length(fieldnames(PAR))>length(names)
%   keyboard
% end
% PAR = orderfields(PAR,defaults);

new_field = orderfields(old_struct,defaults);

return

function SETs = make_partitions(SETs)
 

if SETs.num_partitions==1
  return
end
  
SETs.partitions = eye(SETs.num_partitions,SETs.num_partitions)+1;  
for i = 1:2:SETs.num_partitions-1
  SETs.partitions(i:i+1,i:i+1) = SETs.partitions(i:i+1,i:i+1)+1;
end
if mod(SETs.num_partitions,2)
  SETs.partitions(end,end-1:end) =  SETs.partitions(end,end-1:end)+1;
end
if SETs.num_splits~=SETs.num_partitions
  assert(mod(SETs.num_splits,SETs.num_partitions)==0)
  SETs.partitions = repmat(SETs.partitions,1,SETs.num_splits/SETs.num_partitions);
end

if ~isfield(SETs,'partition_train_on') 
  [tmp,idx] = unique(SETs.partitions==1,'rows');
  SETs.partition_train_on = zeros(1,SETs.num_partitions);
  SETs.partition_train_on(idx) = 1;
end



SETs.info.train = 1;
SETs.info.eval = 2;
SETs.info.plif = 2;
SETs.info.pred = 3;

for i=1:SETs.num_partitions
  assert(isequal(unique(SETs.partitions(i,:)),unique([SETs.info.train ...
                      SETs.info.eval SETs.info.plif SETs.info.pred ])))
end

assert(all(sum(SETs.partitions==SETs.info.pred)==1))
assert(length(unique(sum(SETs.partitions'==SETs.info.train)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.eval)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.plif)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.pred)))==1)




return

function [organism label_source] = dummy_organism(label_source_orig,signal_names)

[label_source] = organism_template(label_source_orig,signal_names);
  
organism.clade = [];
organism.name = '';
organism.full_name = '';
organism.release = '';
organism.version = '';
organism.genebuilt = '';


return

function [label_source] = organism_template(label_source_orig,signal_names)


% PAR = set_label_source(PAR)
% 
% sets defaults for all label_sources (don't use anno, use EST confirmed only.)
% values that are already assigned will NOT be overriden.
  
  
%% default values for general use of annotation and EST confrimed sequences
label_source = label_source_orig;

if isempty(label_source.use_anno)
  label_source.use_anno = 0;
end
if isempty(label_source.use_confirmed)
  label_source.use_confirmed = 1;
end


%% default values for individual signals


for j=1:length(signal_names)
  signal_name = signal_names{j};
  if label_source.use_anno
    label_source.(signal_name).anno = 1;
  else
    label_source.(signal_name).anno = 0;
  end
  if label_source.use_confirmed & ~isequal(signal_name,'transacc')
    label_source.(signal_name).est_clusters = 1;
    label_source.(signal_name).cDNA = 1;
    label_source.(signal_name).fulllength = 1;
  elseif ~label_source.use_confirmed & ~isequal(signal_name,'transacc')
    label_source.(signal_name).est_clusters = 0;
    label_source.(signal_name).cDNA = 0;
    label_source.(signal_name).fulllength = 0;
  end
  label_source.(signal_name).db = 0;
  label_source.(signal_name).from_fn_candsites = 1 ;
end

label_source.tis.maxorf = 0;
label_source.cdsStop.maxorf = 0;

label_source.transacc.anno_SL1 = 0;
label_source.transacc.anno_SL2 = 0;
label_source.transacc.pred = 1;
label_source.transacc.db = 0;



return

function method = SVM_WD(Signal)

method = SVM(Signal);
method.feature_type = 'char';

kernel.name = 'WD';
kernel.order = Signal.order;
kernel.shift = 0 ;
kernel.lwin = Signal.lwin{1};
kernel.rwin = Signal.rwin{1};

method.kernels{1} = kernel; 


return

function method = SVM(Signal)
  
method = method_template;

% set parameters, model selection 

method.name = 'SVM';
method.train_fct = 'svm_train';
method.predict_fct = 'svm_predict';
method.kernels = {[]};
method.poim_maxOrder = 8;
method.svm_name = 'LIGHT';
method.qpsize = 50;
method.threads = 16;
method.center = Signal.lwin_big;
method.par_ms.C = Signal.C;

%PAR.method.svm_name = 'GPBTSVM';PAR.method.qpsize=500;

% use multi kernel learning
method.use_mkl = 0;





return

function Content = polya_tail

name = 'polya_tail';

Content = content_template(name);

Content.wordlen = { [3:5]};

return

function FN=init_FN(signal_names,content_names)
 
  
if nargin<1
  signal_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
end
if nargin<2
  content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
end
 
%--------------------------------------------------------------------------
% Directories
% -------------------------------------------------------------------------------------

FN.genome_dir= '/fml/ag-raetsch/share/databases/genomes/';
FN.input_dir= '/fml/ag-raetsch/share/projects/genefinding/';
FN.output_dir = '/fml/ag-raetsch/share/projects/genefinding/';

FN.dir_name = '';
FN.exp_name = '';
FN.lsl_exp_name = '';
%--------------------------------------------------------------------------
% INPUT FILE NAMES
% -------------------------------------------------------------------------------------

FN.genome.fn_confirmed_sequences = 'confirmed_sequences.mat';
FN.genome.fn_genome_config_orig  = 'genome_uncondensed.config';
FN.genome.fn_genome_config  = 'genome.config';
FN.genome.fn_map = 'condense_map.txt';
FN.genome.fn_genes_anno = 'annotated_genes.mat';
FN.genome.fn_genes_merged = '';
FN.genome.fn_genes_conf = 'confirmed_sequences.mat';
FN.genome.fn_name_to_contig_mapping = 'sequences/contig_orig_new_mapping';


FN.input.fn_genome_config = 'genome.config';
FN.input.fn_trainings_regions = 'training_regions.mat';
FN.input.fn_test_regions = 'test_regions.mat';
FN.input.fn_genes_anno_orig ='annotated_genes.mat';
FN.input.fn_genes_conf_orig = 'confirmed_sequences.mat';
FN.input.fn_ESTs_matches = {'est_matches.mat'};

%--------------------------------------------------------------------------
% OUTPUT FILE NAMES
% -------------------------------------------------------------------------------------

FN.output.fn_genes_anno ='anno_genes.mat';
FN.output.fn_genes_conf = 'conf_genes.mat';
FN.output.fn_genes_merged = 'genes_all.mat';
FN.output.fn_genes_signals = 'genes_signals.mat';
FN.output.fn_genes_contents = 'annotated_genes.mat';
FN.output.fn_genes_lsl = 'annotated_genes.mat';
FN.output.fn_ESTs_matches = [];
FN.output.fn_trivial_regions = 'trivial_regions.mat';
FN.output.fn_blocks = 'blocks.mat';
FN.output.fn_training_blocks ='training_blocks.mat';
FN.output.fn_test_blocks = 'test_blocks.mat';

%--------------------------------------------------------------------------
% SIGNAL FILE NAMES
% -------------------------------------------------------------------------------------

for j=1:length(signal_names)
  sig_name = signal_names{j};
  FN.input_sig.(sig_name).fn_candsites = 'cands/' ;
  FN.input_sig.(sig_name).fn_examples = 'examples';
  FN.input_sig.(sig_name).fn_pos = 'pos/';
  FN.input_sig.(sig_name).fn_filter_settings= 'filter_settings.mat';
  FN.input_sig.(sig_name).fn_example_statistics = 'example_statistics.mat';

  FN.output_sig.(sig_name).fn_models='MS/models';
  FN.output_sig.(sig_name).fn_SVMs='MS/';
  FN.output_sig.(sig_name).fn_pred='pred/';
  FN.output_sig.(sig_name).fn_motifs='MS/motifs';
end


if any(strcmp(signal_names,'tss'))
  FN.input_sig.tss.fn_est_cluster = 'tss_est_clusters.mat';
  FN.input_sig.tss.fn_fulllength  = 'tss_fulllength.mat';
  FN.input_sig.tss.fn_db = 'tss_db.mat' ;
end

if any(strcmp(signal_names,'cleave'))
  FN.input_sig.cleave.fn_est_cluster = 'cleave_est_clusters.mat';
  FN.input_sig.cleave.fn_fulllength = 'cleave_fulllength.mat';
  FN.input_sig.cleave.fn_db  = 'cleave_db.mat';
  FN.input_sig.cleave.fn_anno  = 'cleave_anno.mat';
end
if any(strcmp(signal_names,'polya'))
  FN.input_sig.polya.fn_polya = 'truesites_and_consensus.mat';
  FN.input_sig.polya.fn_log = 'log';
end
if any(strcmp(signal_names,'tis'))
  FN.input_sig.tis.fn_anno = 'tis_anno.mat';
end
if any(strcmp(signal_names,'cdsStop'))
  FN.input_sig.cdsStop.fn_anno = 'cdsStop_anno.mat';
end

if any(strcmp(signal_names,'transacc'))
  FN.input_sig.transacc.fn_anno_SL1 = 'transacc_anno_SL1' ;
  FN.input_sig.transacc.fn_anno_SL2 = 'transacc_anno_SL2' ;
  FN.input_sig.transacc.fn_pred = 'transacc_pred';
  FN.input_sig.transacc.fn_db = []; % only for C_elegans
end

%--------------------------------------------------------------------------
% CONTENT FILE NAMES
% -------------------------------------------------------------------------------------

for j=1:length(content_names)
  content_name = content_names{j};
  FN.input_cont.(content_name) = struct;
  FN.input_cont.(content_name).fn_candsites = 'cands/' ;
  FN.input_cont.(content_name).fn_examples = 'examples';
  FN.input_cont.(content_name).fn_pos = 'pos/';
  FN.input_cont.(content_name).fn_filter_settings= 'filter_settings.mat';
  FN.input_cont.(content_name).fn_example_statistics = 'example_statistics.mat';

  FN.output_cont.(content_name).fn_models = 'MS/models';
  FN.output_cont.(content_name).fn_SVMs = 'MS/';
  FN.output_cont.(content_name).fn_pred = 'pred/';
  FN.output_cont.(content_name).fn_motifs = 'MS/motifs';
end


%--------------------------------------------------------------------------
% LSL FILE NAMES
% -------------------------------------------------------------------------------------

FN.input_lsl.fn_test_blocks = 'data/test_blocks';
FN.input_lsl.fn_training_blocks =  'data/training_blocks.mat';
FN.input_lsl.fn_training_split =  'data/blocks_split.mat';

FN.input_lsl.fn_val_blocks =  'data/validation_blocks.mat';
FN.input_lsl.fn_blocks_all =  'data/blocks_all.mat';
FN.input_lsl.fn_boundary_model = 'data/boundary_model.mat';


FN.output_lsl.fn_init =  'train/init.mat';
FN.output_lsl.fn_train_block_preds = 'train/single_block_predictions/';
FN.output_lsl.fn_train = 'train/train';
FN.output_lsl.fn_pred = 'predictions/';
FN.output_lsl.fn_blocks_pred = 'predictions/blocks_pred';
FN.output_lsl.fn_pred_genome = 'genomewide_predictions/';
FN.output_lsl.fn_log = 'log_files/log_';



return

function FN = create_filenames(FN,organism,Signals,Contents,verb)

%
% FN requires fields :
% 	- input_dirC_
% 	- output_dir
%	- exp_name
%	- FN.fn_genes_train
%	- date
%	- user
%	- organism
%

if nargin<2
  verb=0;
end

if ~isfield(FN,'dir_name_ext')
  dir_name_ext = '';
else
  dir_name_ext = [ '_', FN.dir_name_ext ];
end;

if isempty(FN.dir_name)
  FN.dir_name ='';
end
if isempty(FN.exp_name)
  FN.exp_name = '';
end
if isempty(FN.lsl_exp_name)
  FN.lsl_exp_name = '';
end

assert(isfield(FN,'exp_name'))
assert(isfield(FN,'dir_name'))

%%%%%%%%%%%%%%%
%%% GENERAL INPUT GENOME FILES 
%%%%%%%%%%%%%%%
exp_base_in = sprintf('%s/%s/%s/%s',FN.genome_dir,organism.name,organism.release,organism.genebuilt);

input_fns = fieldnames(FN.genome);
for i=1:length(input_fns)
  old_name = getfield(FN.genome,input_fns{i});
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.genome_dir,old_name{j}))
        new_name{j}  = sprintf('%s/%s',exp_base_in,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  elseif length(strfind(old_name,'/'))==1
    exp_base_in_ = sprintf('%s/%s/%s',FN.genome_dir,organism.name,organism.release);
    new_name  = sprintf('%s/%s',exp_base_in_,old_name );
  elseif isempty(strmatch(FN.genome_dir,old_name)) 
    new_name  = sprintf('%s/%s',exp_base_in,old_name );
  else
    new_name  = old_name ;
  end
  FN.genome = setfield(FN.genome,input_fns{i},new_name);
end  
if verb
  fprintf(1, 'genebuild from dir: \n%s\n\n', exp_base_in)
end

%%%%%%%%%%%%%%%
%%% GENERAL INPUT FILES 
%%%%%%%%%%%%%%%

exp_base_in = sprintf('%s/%s/%s',FN.input_dir,organism.name,FN.dir_name);

input_fns = fieldnames(FN.input);
for i=1:length(input_fns)
  old_name = getfield(FN.input,input_fns{i});
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.input_dir,old_name{j}))
        new_name{j}  = sprintf('%s/orig_data/%s',exp_base_in,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  else
    if isempty(strmatch(FN.input_dir,old_name)) 
      new_name  = sprintf('%s/orig_data/%s',exp_base_in,old_name );
    else
      new_name  = old_name ;
    end
  end
  FN.input = setfield(FN.input,input_fns{i},new_name);
end  
input_dir=sprintf('%s/orig_data/',exp_base_in);
if verb
  fprintf(1, 'dir for original data: \n%s\n\n', input_dir)
end


%%%%%%%%%%%%%%%%%%%%%%%%%%
%% OUTPUT DIRs AND FNs  
%%%%%%%%%%%%%%%%%%%%%%%%%
exp_base_out = sprintf('%s/%s/%s',FN.output_dir,organism.name,FN.dir_name);

output_fns = fieldnames(FN.output);
for i=1:length(output_fns)
  old_name = getfield(FN.output,output_fns{i});
  if isempty(old_name)
    old_name = '';
  end 
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.output_dir,old_name{j})) 
        new_name{j}  = sprintf('%s/genome_data/%s',exp_base_out,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  else
    if  isempty(strmatch(FN.output_dir,old_name)) 
      new_name  = sprintf('%s/genome_data/%s',exp_base_out,old_name );
    else
      new_name  = old_name ;
    end
  end
  FN.output = setfield(FN.output,output_fns{i},new_name);
end  

output_dir=sprintf('%s/genome_data/',exp_base_out);
if verb
  fprintf(1, 'dir for genes, split and regions: \n%s\n\n', output_dir)
end
if ~isfield( FN,'exp_dir')||isempty(FN.exp_dir)|| isempty(strmatch(FN.input_dir,FN.exp_dir)) 
  FN.exp_dir = sprintf('%s/%s',exp_base_out,FN.exp_name);
end

%%%%%%%%%%%%%%%
%%% SIGNAL SPECIFIC INPUT FILES 
%%%%%%%%%%%%%%%

signalnames = fieldnames(FN.input_sig);
for s=1:length(signalnames)
  signals = getfield(FN.input_sig,signalnames{s});
  signal_fns = fieldnames(signals);
  for i=1:length(signal_fns)
    old_name = getfield(signals,signal_fns{i}) ;
    if isempty(old_name)
       old_name = '';
    end 
    if isempty(strmatch(FN.input_dir,old_name)) 
      new_name  = sprintf('%s/%s/%s/%s',exp_base_in,FN.exp_name,dir_name_ext,old_name );
      % new_name  = sprintf('%s/%s/sensors/%s%s/%s',exp_base_in,FN.exp_name,signalnames{s},dir_name_ext,old_name );
    else
      new_name  = old_name ; 
    end
    signals = setfield(signals,signal_fns{i},new_name);
  end 
  FN.input_sig = setfield(FN.input_sig,signalnames{s},signals);
  
  % sig_input = sprintf('%s/%s/sensors/%s%s',exp_base_in,FN.exp_name,signalnames{s},dir_name_ext,old_name);
  if verb
    fprintf(1, 'dir for label_files and examples: \n%s\n\n', new_name)
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SIGNAL SPECIFIC OUTPUT DIRs AND FNs  
%%%%%%%%%%%%%%%%%%%%%%%%%



%output_fns = fieldnames(FN.output);
%for i=1:length(output_fns)
%  old_name = getfield(FN.output,output_fns{i});
%  new_name  = sprintf('%s/%s_%s/%s',FN.output_dir,FN.exp_name,...
%                      organism.name,old_name );
%  FN.output = setfield(FN.output,output_fns{i},new_name);
%end  

signalnames = fieldnames(FN.output_sig);
for s=1:length(signalnames)
  signals = FN.output_sig.(signalnames{s});
  signal_fns = fieldnames(signals);
  method = Signals.(signalnames{s}).method;
  for i=1:length(signal_fns)
    old_name = getfield(signals,signal_fns{i}) ;
    dir_name = sprintf('%s/%s/%s',exp_base_out,FN.exp_name,dir_name_ext);
    % dir_name = sprintf('%s/%s/sensors/%s%s',exp_base_out,FN.exp_name,signalnames{s},dir_name_ext);
    if isfield(FN, 'custom_output')
      dir_name = sprintf('%s/%s',dir_name,FN.custom_output);
    else
      dir_name = sprintf('%s/output_%s',dir_name,method.name);
      if isequal(method.name, 'SVM')
        if length(method.kernels)==1&&~isempty(method.kernels{1})
          if isfield(method.kernels{1},'name')
            kernel_names = method.kernels{1}.name; 
          else
            kernel_names = method.kernels{1}.kernel_name;
          end
        else
          kernel_names = 'COMBINED';
        end
        dir_name = sprintf('%s%s',dir_name,kernel_names);
      end
    end
    if isempty(strmatch(FN.output_dir,old_name)) 
      new_name  = sprintf('%s/%s',dir_name, old_name );
    else
      new_name = old_name;
    end
    signals = setfield(signals,signal_fns{i},new_name);
  end
  FN.output_sig = setfield( FN.output_sig, signalnames{s}, signals );
  if verb
    fprintf(1, 'dir for training and test data: \n%s\n\n', dir_name) 
  end
end


%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC INPUT FILES 
%%%%%%%%%%%%%%%

if isfield(FN, 'input_cont')
  cont_names = fieldnames(FN.input_cont);
  for s=1:length(cont_names)
    content = getfield(FN.input_cont,cont_names{s});
    cont_fns = fieldnames(content);
    for i=1:length(cont_fns)
      old_name = getfield(content,cont_fns{i}) ;
      if isempty(strmatch(FN.input_dir,old_name)) 
        new_name  = sprintf('%s/%s/%s/%s',exp_base_in,FN.exp_name,dir_name_ext,old_name );
        % new_name  = sprintf('%s/%s/contents/%s%s/%s',exp_base_in,FN.exp_name, cont_names{s},dir_name_ext,old_name );
      else
        new_name  = old_name ; 
      end
      content = setfield(content,cont_fns{i},new_name);
    end 
    FN.input_cont = setfield(FN.input_cont,cont_names{s},content);
  end

  %cont_input = sprintf('%s/%s/contents/%s',exp_base_in,FN.exp_name,signal_dir_name);
  if verb
    fprintf(1, 'dir for content specific input: \n%s\n\n', cont_input)
  end
end

%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC OUTPUT FILES 
%%%%%%%%%%%%%%%

if isfield(FN, 'output_cont')
  cont_names = fieldnames(FN.output_cont);
  for s=1:length(cont_names)
    content = getfield(FN.output_cont,cont_names{s});
    cont_fns = fieldnames(content);
    method = Contents.(cont_names{s}).method;
    for i=1:length(cont_fns)
      old_name = getfield(content,cont_fns{i}) ;
      dir_name = sprintf('%s/%s/%s',exp_base_out,FN.exp_name,dir_name_ext);
      if isfield(FN, 'custom_output')
        dir_name = sprintf('%s/%s',dir_name,FN.custom_output);
      else
        dir_name = sprintf('%s/output_%s',dir_name,method.name);
        if isequal(method.name, 'SVM')
          if length(method.kernels)==1&&~isempty(method.kernels{1})
            if isfield(method.kernels{1},'name')
              kernel_names = method.kernels{1}.name; 
            else
              kernel_names = method.kernels{1}.kernel_name;
            end
          else
            kernel_names = 'COMBINED';
          end
          dir_name = sprintf('%s%s',dir_name,kernel_names);
        end
      end
      if isempty(strmatch(FN.output_dir,old_name)) 
        new_name  = sprintf('%s/%s',dir_name, old_name );
      else
        new_name = old_name;
      end
      content = setfield(content,cont_fns{i},new_name);
    end 
    FN.output_cont = setfield(FN.output_cont, cont_names{s},content);
  end

  %cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
  if verb
    fprintf(1, 'dir for content specific output: \n%s\n\n', cont_output)
  end
end

%%%%%%%%%%%%%%%
%%% LSL STUFF
%%%%%%%%%%%%%%%

if isfield(FN, 'input_lsl')
  fns = fieldnames(FN.input_lsl);
  FN.input_lsl.dir = sprintf('%s/%s/%s/%s/lsl/%s/', ...
			     exp_base_out,organism.name,FN.dir_name,FN.exp_name,FN.lsl_exp_name ); 
  for i=1:length(fns)
    if ~iscell(FN.input_lsl.(fns{i}))
      old_name = FN.input_lsl.(fns{i}) ;
      if isempty(strmatch(FN.input_lsl.dir,old_name))&~isequal(fns{i},'fn_old_training_blocks')...
            &~isequal(fns{i},'fn_old_test_blocks')&~isequal(fns{i},'fn_old_split')
        new_name  = sprintf('%s/%s',FN.input_lsl.dir,old_name);
      else
        new_name  = old_name ; 
      end
      FN.input_lsl.(fns{i}) = new_name;
    else
      for j=1:length(FN.input_lsl.(fns{i}))
        old_name = FN.input_lsl.(fns{i}){j} ;
        if isempty(strmatch(FN.input_lsl.dir,old_name))&~isequal(fns{i},'fn_old_training_blocks')...
              &~isequal(fns{i},'fn_old_test_blocks')&~isequal(fns{i},'fn_old_split')
          new_name  = sprintf('%s/%s',FN.input_lsl.dir,old_name);
        else
          new_name  = old_name ; 
        end
        FN.input_lsl.(fns{i}){j} = new_name;
      end    
    end
  end
end
%cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
if verb
  fprintf(1, 'dir for lsl specific input: \n%s\n\n', cont_output)
end

if isfield(FN, 'output_lsl')
  fns = fieldnames(FN.output_lsl);
  FN.output_lsl.dir = sprintf('%s/%s/%s/%s/lsl/%s/', ...
			      exp_base_out, organism.name,FN.dir_name,FN.exp_name,FN.lsl_exp_name ); 
  for i=1:length(fns)
    old_name = FN.output_lsl.(fns{i}) ;
    if isempty(strmatch(FN.output_lsl.dir,old_name))
      new_name  = sprintf('%s/%s',FN.output_lsl.dir,old_name);
    else
      new_name  = old_name ; 
    end
    FN.output_lsl.(fns{i}) = new_name;
  end 
end
%cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
if verb
  fprintf(1, 'dir for lsl specific output: \n%s\n\n', cont_output)
end


return

function [model] = make_model(organism,LSL,Signals,old)
% [model] = make_model(bins)

if nargin<4
  old=0;
end
  
model.bins = LSL.method.plif_bins ;
% model.use_gc =  LSL.method.use_gc;
model.use = LSL.method.use;
polya_sixmers = Signals.polya.consensus;

model = define_segments(model);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%define features

model = define_contents(model);
model = define_lengths(model,organism);
model = define_signals(model,polya_sixmers);
model = define_addContentFeatures(model);

model.cnt_plifs = model.cnt_contents+model.cnt_lengths+model.cnt_signals+model.cnt_tiling;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% remember the ordering of these fields
model.segments_fields = fieldnames(model.segments) ;
model.contents_fields = fieldnames(model.contents) ;
model.lengths_fields = fieldnames(model.lengths) ;
model.signals_fields = fieldnames(model.signals) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%define states

model = define_states(model);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define mappings between length, contents and segments

model = define_mappings(model);
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model = gen_transition_matrix(model) ;
% transition_pointers used to be called penalties
model = define_transition_pointers(model);
if old
  model.penalties = model.transition_pointers;
end
model.A(~isinf(model.A))=0 ;



model.cnt_parameters = model.cnt_transitions + model.cnt_plifs*model.bins ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% define loss 
model =  define_loss(model);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% mappings of PLIFS to parameters

if ~old
  map_plif2param = init_weights(model);
  map_plif2param = rmfield(map_plif2param,'transitions');
  weights_name = fieldnames(map_plif2param);
  start=0;
  for w=1:length(weights_name)
    len=length(getfield(map_plif2param, weights_name{w}));
    model.plif2param{w} = start+(1:len);
    start = start + len;
  end
  model.transitions2param = start+1:model.cnt_parameters ;
  
  assert(model.cnt_parameters-model.cnt_transitions==start)
end

return

function model = define_segments(model)
% declare segment types

  cnt=0;  
model.segments.intergenic = cnt ;cnt=cnt+1;
model.segments.utr5exon   = cnt ;cnt=cnt+1;
if model.use.segments.transexon 
  model.segments.transexon  = cnt ;cnt=cnt+1;
end
model.segments.cds_exon   = cnt ;cnt=cnt+1;
model.segments.utr3exon   = cnt ;cnt=cnt+1;
if model.use.segments.polya_tail
  model.segments.polya_tail = cnt ;cnt=cnt+1;
end
if model.use.segments.intercistronic
  model.segments.intercistronic  = cnt ;cnt=cnt+1;
  model.segments.intergenictrans  = cnt ;cnt=cnt+1;
end
model.segments.intron     = cnt ;cnt=cnt+1;

model.cnt_segments = length(fieldnames(model.segments));


return

function model = define_contents(model)
  
% declare content types

fid = 1;  
model.mod_words = [];
% the first contents consider all k-mers. the last three
% frame-content sensors only consider codons, with an offset from
% the end of the segment

content_start =  1 ;
cnt=0;

model.contents.intergenic = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.utr_exon   = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.cds_exon   = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.intron     = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
if model.use.segments.intercistronic
  model.contents.intercistronic = content_start+cnt;cnt=cnt+1;
  model.mod_words = [model.mod_words; 1 0] ;
end
if model.use.contents.frame
  model.contents.cds_frame0   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 0] ;
  model.contents.cds_frame1   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 1] ;
  model.contents.cds_frame2   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 2] ;
end
model.cnt_contents = length(fieldnames(model.contents));

%hack
if model.cnt_contents<8
  fprintf(fid,'introducing dummy contents\n')
  for i= 1:8-model.cnt_contents
    if i==1
      model.contents.('dummy')= content_start+cnt;cnt=cnt+1;
    else
      model.contents.(['dummy' num2str(i)])= content_start+cnt;cnt=cnt+1;
    end
  end
  model.cnt_contents = length(fieldnames(model.contents));
  model.mod_words = [model.mod_words; 1 0] ;
end

% declare content range
model.contents_range = struct;
content_names = fieldnames(model.contents) ;
for s = 1:length(content_names)
  model.contents_range = setfield(model.contents_range, content_names{s},[-10,10]);
end


return

function model = define_lengths(model,organism)
  
fid = 1; 
% declare length types
length_start = model.cnt_contents+1 ;
cnt = 0;

model.lengths.intergenic = length_start+cnt; cnt = cnt+1;
model.lengths.utr5exon = length_start+cnt ;cnt = cnt+1;
if model.use.segments.transexon 
  model.lengths.transexon = length_start+cnt ;cnt = cnt+1;
end
model.lengths.single_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.first_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.middle_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.last_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.utr3exon = length_start+cnt;cnt = cnt+1;
if model.use.segments.polya_tail 
  model.lengths.polya_tail = length_start+cnt;cnt = cnt+1;
end
model.lengths.intron = length_start+cnt ;cnt = cnt+1;
if model.use.segments.intercistronic
  model.lengths.intercistronic = length_start+cnt ;cnt = cnt+1;
  model.lengths.intergenictrans = length_start+cnt ;cnt = cnt+1;
end
model.lengths.intergenic_long = length_start+cnt ;cnt = cnt+1;

model.cnt_lengths = length(fieldnames(model.lengths));


% declare length range
s1=which([organism.name '_lengths']);
s2=which([organism.clade '_lengths']);
if ~isempty(s1)
  fprintf(fid,'taking lengths range from file: %s_lengths.m \n',organism.name)
  model.lengths_range = feval([organism.name '_lengths']);
elseif ~isempty(s2)
  fprintf(fid,'taking lengths range from file: %s_lengths.m \n',organism.clade)
  model.lengths_range = feval([organism.clade '_lengths']);
else
  fprintf(fid,'no lengths range specified, using default settings\n')
  model.lengths_range = feval('default_lengths');
end

return

function model = define_signals(model,polya_sixmers)
  
% declare signal types
cnt=1;
signal_start = model.cnt_contents + model.cnt_lengths ;

if model.use.signals.tss
  model.signals.tss = signal_start+cnt;
  model.consensus.tss = {} ;
  model.consensus_pos.tss = [];
  cnt=cnt+1;
end

model.signals.tis = signal_start+cnt;
model.consensus.tis = {'atg'} ;
model.consensus_pos.tis = [0];
cnt=cnt+1;

model.signals.don = signal_start+cnt;
model.consensus.don = {'gt','gc'} ;
model.consensus_pos.don = [0,0];
cnt=cnt+1;

model.signals.acc = signal_start+cnt;
model.consensus.acc = {'ag'} ;
model.consensus_pos.acc = [2];
cnt=cnt+1;

model.signals.cdsStop = signal_start+cnt ;
model.consensus.cdsStop = {'taa','tga','tag'} ;
model.consensus_pos.cdsStop = [0,0,0];
cnt=cnt+1;

if model.use.signals.polya
  model.signals.polya = signal_start+cnt ;
  model.consensus.polya = polya_sixmers;
  model.consensus_pos.polya = zeros(1,length(polya_sixmers)) ;
  cnt=cnt+1;
end
if model.use.signals.cleave
  model.signals.cleave = signal_start+cnt ;
  model.consensus.cleave = {} ;
  model.consensus_pos.cleave = [];
  cnt=cnt+1;
end

if model.use.signals.trans
  model.signals.trans = signal_start+cnt;
  model.consensus.trans = {'ag'} ;
  model.consensus_pos.trans = [2];
  cnt=cnt+1;
end
if model.use.signals.transacc
  model.signals.transacc = signal_start+cnt;
  model.consensus.transacc = {'ag'} ;
  model.consensus_pos.transacc = [2];
  cnt=cnt+1;
end

model.cnt_signals = length(fieldnames(model.signals));


% declare signal ranges for PLiFs

if model.use.signals.tss
  model.signals_range.tss = [0,1] ;
end
model.signals_range.tis = [0,1] ;
model.signals_range.don = [0,1] ;
model.signals_range.acc = [0,1] ;
model.signals_range.cdsStop = [0,1] ;
if model.use.signals.polya
  model.signals_range.polya = [0,1] ;
end
if model.use.signals.cleave
  model.signals_range.cleave = [0,1] ;
end
if model.use.signals.trans
  model.signals_range.trans = [0,1] ;
end
if model.use.signals.transacc
  model.signals_range.transacc = [0,1] ;
end



return

function model = define_addContentFeatures(model)
  
% plifs for tiling array data
if model.use.addfeatures.tiling
  tiling_start =  model.cnt_contents + model.cnt_lengths + model.cnt_signals;
  content_names = fieldnames(model.contents);
  model.tiling_plif_ids = [];
  for s = 1:length(content_names)
    model.tiling.(content_names{s}) = tiling_start+s ;
    model.tiling_plif_ids = [model.tiling_plif_ids tiling_start+s];
  end
  model.cnt_tiling = length(fieldnames(model.tiling));
  % declare range
  model.tiling_range = struct;
  tiling_names = fieldnames(model.tiling) ;
  for s = 1:length(tiling_names)
    model.tiling_range = setfield(model.tiling_range, tiling_names{s},[6,11]);% ca. 2%-98% Percentile
  end
else
  model.cnt_tiling = 0;
end

return

function model = define_states(model)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define states
model.states(1) = struct ;
cnt=0 ;

% sequence start 
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_start' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_start=cnt ;

signal_names = fieldnames(model.signals) ;
for i=1:model.cnt_signals
  if strcmp(signal_names{i}, 'transacc'), continue; end ;
  if ~strcmp(signal_names{i},'acc') && ~strcmp(signal_names{i},'don') ...
        && ~strcmp(signal_names{i}, 'tis') && ~strcmp(signal_names{i}, 'cdsStop')...
        && ~strcmp(signal_names{i}, 'trans') && ~strcmp(signal_names{i}, 'cleave') 
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    model.state_ids=setfield(model.state_ids, signal_names{i},cnt) ;
  elseif strcmp(signal_names{i},'cleave') 
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    model.state_ids=setfield(model.state_ids, signal_names{i},[cnt]) ;
    if model.use.segments.intercistronic
      cnt=cnt+1 ;
      model.states(cnt).name = signal_names{i};
      model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
      model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
      model.states(cnt).non_consensus = {} ;
      model.states(cnt).non_consensus_pos = [] ;
      model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
      model.state_ids=setfield(model.state_ids, signal_names{i},[cnt-1 cnt]) ;
    end
  elseif strcmp(signal_names{i},'tis') | strcmp(signal_names{i},'cdsStop')
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.state_ids=setfield(model.state_ids, signal_names{i},[cnt-1 cnt]) ;
    if strcmp(signal_names{i}, 'tis')
    model.states(cnt).name = 'tis_acc' ;
      model.states(cnt).consensus = {'agatg'} ;
      model.states(cnt).consensus_pos = 2 ;
      model.states(cnt).signal = [model.signals.acc model.signals.tis] ;%%%% !!! 2 Zahlen
    else
      model.states(cnt).name = 'cdsStop_acc';
      model.states(cnt).consensus = {'agtaa', 'agtag', 'agtga'} ;
      model.states(cnt).consensus_pos = [2 2 2] ;
      model.states(cnt).signal = [model.signals.acc model.signals.cdsStop] ;%%%% !!! 2 Zahlen
    end
  elseif strcmp(signal_names{i},'trans') & model.use.states.SL1 & model.use.states.SL2
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL1';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL1';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;
    model.state_ids = setfield(model.state_ids, 'SL1', [cnt-1 cnt]) ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL2';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL2';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;%%%% !!! 3 Zahlen
    model.state_ids = setfield(model.state_ids, 'SL2', [cnt-1 cnt]) ;
  elseif strcmp(signal_names{i},'trans') & ~model.use.states.SL1 & ~model.use.states.SL2
    cnt=cnt+1 ;
    model.states(cnt).name = 'trans';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'trans';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;
    model.state_ids = setfield(model.state_ids, 'SL1', [cnt-1 cnt]) ;
  elseif strcmp(signal_names{i},'don')
    cnt_don = cnt ;
    cnt=cnt+8 ;
    for id = cnt_don+1:cnt_don+8 ;
      model.states(id).consensus = getfield(model.consensus,signal_names{i});
      model.states(id).consensus_pos = getfield(model.consensus_pos,signal_names{i});
      model.states(id).non_consensus = {} ;
      model.states(id).non_consensus_pos = [] ;
      model.states(id).signal=model.signals.don ;
    end ;
    model.states(cnt_don+1).name='UTR5_don' ;
    model.states(cnt_don+2).name='CDS_don_0' ;
    model.states(cnt_don+3).name='CDS_don_1a' ;
    model.states(cnt_don+3).non_consensus = {'t'} ;
    model.states(cnt_don+3).non_consensus_pos= [1] ;
    model.states(cnt_don+4).name='CDS_don_1b' ;
    model.states(cnt_don+4).consensus = {'tgt','tgc'} ;
    model.states(cnt_don+4).consensus_pos= [1,1] ;
    model.states(cnt_don+5).name='CDS_don_2a' ;
    model.states(cnt_don+5).non_consensus = {'ta', 'tg'} ;
    model.states(cnt_don+5).non_consensus_pos= [2,2] ;
    model.states(cnt_don+6).name='CDS_don_2b' ;
    model.states(cnt_don+6).consensus = {'tagt', 'tagc'} ;
    model.states(cnt_don+6).consensus_pos= [2,2] ;
    model.states(cnt_don+7).name='CDS_don_2c' ;
    model.states(cnt_don+7).consensus = {'tggt', 'tggc'} ;
    model.states(cnt_don+7).consensus_pos= [2,2] ;
    model.states(cnt_don+8).name='UTR3_don' ;
    
    model.state_ids.don=cnt_don+1:cnt_don+8 ;
  elseif strcmp(signal_names{i},'acc')
    cnt_acc = cnt ;
    cnt=cnt+8 ;
    for id = cnt_acc+1:cnt_acc+8 ;
      model.states(id).consensus = {'ag'} ;
      model.states(id).consensus_pos= [2] ;
      model.states(id).non_consensus={} ;
      model.states(id).non_consensus_pos=[] ;
      model.states(id).signal=model.signals.acc ;
    end ;
    model.states(cnt_acc+1).name='UTR5_acc' ;
    model.states(cnt_acc+2).name='CDS_acc_0' ;
    model.states(cnt_acc+3).name='CDS_acc_1a' ;
    model.states(cnt_acc+4).name='CDS_acc_1b' ;
    model.states(cnt_acc+4).non_consensus = {'aa','ag','ga'} ;
    model.states(cnt_acc+4).non_consensus_pos= [0,0,0] ;
    model.states(cnt_acc+5).name='CDS_acc_2a' ;
    model.states(cnt_acc+6).name='CDS_acc_2b' ;
    model.states(cnt_acc+6).consensus = {'agt', 'agc'} ;
    model.states(cnt_acc+6).consensus_pos= [2, 2] ;
    model.states(cnt_acc+7).name='CDS_acc_2c' ;
    model.states(cnt_acc+7).non_consensus = {'a'} ;
    model.states(cnt_acc+7).non_consensus_pos= [0] ;
    model.states(cnt_acc+8).name='UTR3_acc' ;
 
    model.state_ids.acc=cnt_acc+1:cnt_acc+8 ;

    if isfield(model.use, 'intron_long_state')&& model.use.intron_long_state
      cnt_intron_long = cnt;
      cnt = cnt+6;
      model.states(cnt_intron_long+1).consensus={} ;
      model.states(cnt_intron_long+1).consensus_pos=[] ;
      model.states(cnt_intron_long+1).non_consensus={} ;
      model.states(cnt_intron_long+1).non_consensus_pos=[] ;
      model.states(cnt_intron_long+1).name='intron_long_0' ;
      model.states(cnt_intron_long+1).signal=[] ;

      model.states(cnt_intron_long+2).consensus={} ;
      model.states(cnt_intron_long+2).consensus_pos=[] ;
      model.states(cnt_intron_long+2).non_consensus={} ;
      model.states(cnt_intron_long+2).non_consensus_pos=[] ;
      model.states(cnt_intron_long+2).name='intron_long_1a' ;
      model.states(cnt_intron_long+2).signal=[] ;
    
      model.states(cnt_intron_long+3).consensus={} ;
      model.states(cnt_intron_long+3).consensus_pos=[] ;
      model.states(cnt_intron_long+3).non_consensus={} ;
      model.states(cnt_intron_long+3).non_consensus_pos=[] ;
      model.states(cnt_intron_long+3).name='intron_long_1b' ;
      model.states(cnt_intron_long+3).signal=[] ;

      model.states(cnt_intron_long+4).consensus={} ;
      model.states(cnt_intron_long+4).consensus_pos=[] ;
      model.states(cnt_intron_long+4).non_consensus={} ;
      model.states(cnt_intron_long+4).non_consensus_pos=[] ;
      model.states(cnt_intron_long+4).name='intron_long_2a' ;
      model.states(cnt_intron_long+4).signal=[] ;

      model.states(cnt_intron_long+5).consensus={} ;
      model.states(cnt_intron_long+5).consensus_pos=[] ;
      model.states(cnt_intron_long+5).non_consensus={} ;
      model.states(cnt_intron_long+5).non_consensus_pos=[] ;
      model.states(cnt_intron_long+5).name='intron_long_2b' ;
      model.states(cnt_intron_long+5).signal=[] ;

      model.states(cnt_intron_long+6).consensus={} ;
      model.states(cnt_intron_long+6).consensus_pos=[] ;
      model.states(cnt_intron_long+6).non_consensus={} ;
      model.states(cnt_intron_long+6).non_consensus_pos=[] ;
      model.states(cnt_intron_long+6).name='intron_long_1a' ;
      model.states(cnt_intron_long+6).signal=[] ;

      model.state_ids.intergenic_long= cnt_intron_long+1:cnt_intron_long+6;
    end
%    cnt
  end
end

% intergenic_long
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='intergenic_long' ;
model.states(cnt).signal=[] ;
model.state_ids.intergenic_long=cnt ;

% sequence end
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_end' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_end = cnt ;

model.cnt_states = cnt ; 

return

function model = define_mappings(model)
  

%%mappings between length and contents
  
model.plif_links = [] ;
model.plif_links(end+1,:) = [model.lengths.intergenic      model.contents.intergenic] ;
model.plif_links(end+1,:) = [model.lengths.first_cds_exon  model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.last_cds_exon   model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.middle_cds_exon model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.single_cds_exon model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.intron          model.contents.intron] ;
if model.use.segments.transexon
  model.plif_links(end+1,:) = [model.lengths.transexon       model.contents.utr_exon] ;
end
model.plif_links(end+1,:) = [model.lengths.utr5exon        model.contents.utr_exon] ;
model.plif_links(end+1,:) = [model.lengths.utr3exon        model.contents.utr_exon] ;
if model.use.segments.polya_tail
  model.plif_links(end+1,:) = [model.lengths.polya_tail      model.contents.utr_exon] ;
end
if model.use.segments.intercistronic
  model.plif_links(end+1,:) = [model.lengths.intercistronic  model.contents.intercistronic] ;
  model.plif_links(end+1,:) = [model.lengths.intergenictrans  model.contents.intergenic] ;
end
model.plif_links(end+1,:) = [model.lengths.intergenic_long model.contents.intergenic] ;


if isfield(model.state_ids, 'intron_long') 
  model.plif_links(end+1,:) = [model.lengths.intron_long model.contents.intron] ;  
end

%%mappings between length and additional contents (tiling)

if model.use.addfeatures.tiling
  model.tiling_links = [] ;   
  model.tiling_links(end+1,:) = [model.lengths.intergenic      model.tiling.intergenic] ;
  model.tiling_links(end+1,:) = [model.lengths.first_cds_exon  model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.last_cds_exon   model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.middle_cds_exon model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.single_cds_exon model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.intron          model.tiling.intron] ;
  if model.use.segments.transexon
    model.tiling_links(end+1,:) = [model.lengths.transexon       model.tiling.transexon] ;
  end
  model.tiling_links(end+1,:) = [model.lengths.utr5exon        model.tiling.utr_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.utr3exon        model.tiling.utr_exon] ;
  if model.use.segments.polya_tail
    model.tiling_links(end+1,:) = [model.lengths.polya_tail      model.tiling.utr_exon] ;
  end
  model.tiling_links(end+1,:) = [model.lengths.intergenic_long model.tiling.intergenic] ;
  if model.use.segments.intercistronic
    model.tiling_links(end+1,:) = [model.lengths.intercistronic  model.tiling.intercistronic] ;
  end
end

%%mappings between length and segments

model.seg_links = [] ;
model.seg_links(end+1,:) = [model.lengths.intergenic      model.segments.intergenic] ;
model.seg_links(end+1,:) = [model.lengths.first_cds_exon  model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.last_cds_exon   model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.middle_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.single_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.intron          model.segments.intron] ;
if model.use.segments.transexon
  model.seg_links(end+1,:) = [model.lengths.transexon       model.segments.transexon] ;
end
model.seg_links(end+1,:) = [model.lengths.utr5exon        model.segments.utr5exon] ;
model.seg_links(end+1,:) = [model.lengths.utr3exon        model.segments.utr3exon] ;
if model.use.segments.polya_tail
  model.seg_links(end+1,:) = [model.lengths.polya_tail      model.segments.polya_tail] ;
end
if model.use.segments.intercistronic
  model.seg_links(end+1,:) = [model.lengths.intergenictrans  model.segments.intergenictrans] ;
  model.seg_links(end+1,:) = [model.lengths.intercistronic  model.segments.intercistronic] ;
end
model.seg_links(end+1,:) = [model.lengths.intergenic_long model.segments.intergenic] ;
%

return

function model = gen_transition_matrix(model,PAR)

use=model.use;  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define transitions

A = -inf(model.cnt_states, model.cnt_states);
p = -inf(model.cnt_states,1);
q = -inf(model.cnt_states,1) ;
p(model.state_ids.seq_start) = 0 ;
q(model.state_ids.seq_end) = 0 ;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% start constructing transition matrix A

orf_from = -1*ones(model.cnt_states,1) ;
orf_to = -1*ones(model.cnt_states,1) ;

orf_from(model.state_ids.acc(2:7))= [ 0 1 1 2 2 2] ;
orf_from(model.state_ids.tis)     = 0 ; % AGAGT case
if model.use.states.SL1 & model.use.states.SL2
  orf_from(model.state_ids.SL1(2)) = 0 ; % AGAGT case
  orf_from(model.state_ids.SL2(2)) = 0 ; % AGAGT case
elseif model.use.signals.trans
  orf_from(model.state_ids.trans(2)) = 0 ; % AGAGT case
end
orf_to(model.state_ids.don(2:7))  = [0 1 1 2 2 2] ;
orf_to(model.state_ids.cdsStop(1))   = 0 ;
orf_info=[orf_from, orf_to] ;

%%%%%%%%%%%
%% no gene / intergenic stuff
%%%%%%%%%%%
if ~use.model.one_gene,
  A(model.state_ids.seq_start,  model.state_ids.seq_end) = model.lengths.intergenic ;
end
A(model.state_ids.intergenic_long, model.state_ids.intergenic_long) = 0; % no penalty et al
A(model.state_ids.seq_start, model.state_ids.intergenic_long) = model.lengths.intergenic_long ;
A(model.state_ids.intergenic_long, model.state_ids.seq_end) = model.lengths.intergenic_long ;

%%%%%%%%%%%
% 5' UTR 
%%%%%%%%%%%
gene_start_states_in = [];
gene_start_states_out = [];
mRNA_start_states_out = [model.state_ids.tis];

if use.signals.tss,
  gene_start_states_in = [gene_start_states_in model.state_ids.tss] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.tss] ;
  if use.model.allow_missing_tss,
    assert(false) ;
    gene_start_states_in = [gene_start_states_in model.state_ids.tis] ;
  end ;
else
  assert(false)
  gene_start_states_in = [gene_start_states_in model.state_ids.tis] ;
end ;

if use.signals.trans,
  gene_start_states_in = [gene_start_states_in model.state_ids.SL1] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.SL1(1)] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.SL2(1)] ;
  mRNA_start_states_out = [mRNA_start_states_out model.state_ids.SL1(2)] ;%'AGAGT cases'
  mRNA_start_states_out = [mRNA_start_states_out model.state_ids.SL2(2)] ;%'AGAGT cases'
  if use.signals.tss, 
    A(model.state_ids.tss, model.state_ids.SL1)    = model.lengths.transexon ;  
  end ;
  if use.segments.intercistronic & use.signals.cleave ,
    A(model.state_ids.cleave(1), model.state_ids.SL1) = model.lengths.intergenictrans ;
    A(model.state_ids.cleave(2), model.state_ids.SL1) = model.lengths.intercistronic ;
    A(model.state_ids.cleave(2), model.state_ids.SL2) = model.lengths.intercistronic ;
  end ;
end ;

% gene start
A(model.state_ids.seq_start,gene_start_states_in) = model.lengths.intergenic ;
if use.model.use_multi
  A(model.state_ids.intergenic_long, gene_start_states_in) = model.lengths.intergenic_long ; 
end
    
A(gene_start_states_out,model.state_ids.tis(1)) = model.lengths.utr5exon ;
A(gene_start_states_out,model.state_ids.don(1)) = model.lengths.utr5exon ;
% spliced 5'utr
A(model.state_ids.acc(1), model.state_ids.don(1)) = model.lengths.utr5exon ;
A(model.state_ids.don(1), model.state_ids.acc(1)) = model.lengths.intron ;
% last 5'utr exon
A(model.state_ids.acc(1),model.state_ids.tis(1)) = model.lengths.utr5exon ;
A(model.state_ids.don(1), model.state_ids.tis(2)) = model.lengths.intron ; 


%%%%%%%%%%%
% CDS STUFF
%%%%%%%%%%%
A(mRNA_start_states_out, model.state_ids.cdsStop(1)) = model.lengths.single_cds_exon ; % trans(2) ok 
A(mRNA_start_states_out, model.state_ids.don(2:7)) = model.lengths.first_cds_exon ; % trans(2) ok
for i=2:7,
  A(model.state_ids.don(i), model.state_ids.acc(i)) = model.lengths.intron ;
  if isfield(model.state_ids, 'intron_long')
    A(model.state_ids.don(i), model.state_ids.intron_long(i-1)) = model.lengths.intron;
    A(model.state_ids.intron_long(i-1), model.state_ids.acc(i)) = model.lengths.intron_long;
    A(model.state_ids.intron_long(i-1),model.state_ids.intron_long(i-1)) = model.lengths.intron_long;
  end
end ;
A(model.state_ids.acc(2:7), model.state_ids.don(2:7)) = model.lengths.middle_cds_exon ;
A(model.state_ids.acc(2:7), model.state_ids.cdsStop(1)) = model.lengths.last_cds_exon ;
% agstop 
A(model.state_ids.don(2), model.state_ids.cdsStop(2)) = model.lengths.intron ;



%%%%%%%%%%%
% 3' UTR
%%%%%%%%%%%
A(model.state_ids.cdsStop, model.state_ids.don(8)) = model.lengths.utr3exon ;
A(model.state_ids.don(8), model.state_ids.acc(8)) = model.lengths.intron ;
A(model.state_ids.acc(8), model.state_ids.don(8)) = model.lengths.utr3exon ;

if use.signals.polya,
  % polyA has to be in last 3'UTR exon
  A(model.state_ids.cdsStop, model.state_ids.polya) = model.lengths.utr3exon ;
  A(model.state_ids.acc(8), model.state_ids.polya)=model.lengths.utr3exon ;
end ;

gene_stop_states = [];    
if use.signals.cleave,
  gene_stop_states = [gene_stop_states model.state_ids.cleave(1)];
  A(model.state_ids.cdsStop, model.state_ids.cleave) = model.lengths.utr3exon ;
  A(model.state_ids.acc(8), model.state_ids.cleave) = model.lengths.utr3exon ;
  if use.signals.polya,
    A(model.state_ids.polya, model.state_ids.cleave) = model.lengths.polya_tail ;
  end
else
  gene_stop_states = [gene_stop_states model.state_ids.cdsStop];
  if use.signals.polya,
    gene_stop_states = [gene_stop_states model.state_ids.polya];
  end
end ;
A(gene_stop_states, model.state_ids.seq_end)    = model.lengths.intergenic ; 
A(gene_stop_states, model.state_ids.intergenic_long) = model.lengths.intergenic_long ; 
if use.model.use_multi,
  A(gene_stop_states, gene_start_states_in) = model.lengths.intergenic ;    
end ;

if use.segments.intercistronic 
  for k=model.state_ids.SL1,
    idx = find(A(:,k)==model.lengths.intergenic) ;
    A(idx,k) = model.lengths.intergenictrans ;   
  end ;
end
%%%%%%%%%%%
% end transition definition
%%%%%%%%%%%


%%% technical stuff for intergenic_long

transitions = [];
for i=1:length(gene_start_states_in)
  transitions = [transitions; model.state_ids.seq_start gene_start_states_in(i)];
  for j=1:length(gene_stop_states)
    transitions = [transitions; gene_stop_states(j) gene_start_states_in(i)];
    transitions = [transitions; gene_stop_states(j) model.state_ids.seq_end] ;
  end
end

model.intergenic_long.transitions = unique(transitions,'rows');
model.intergenic_long.max_short_length = model.lengths_range.intron(2) ;
model.intergenic_long.step = model.lengths_range.intron(2); % use the intron length as maximal step size
model.intergenic_long.long_segment_parts = model.lengths_range.intron(2)/2-25 ;
%model.intergenic_long.step = 30 ; % makes about 30 (step) * 40 (grid)=1200nt per step

%%% 

q(gene_stop_states) = 0 ;

model.gene_start_states_in = gene_start_states_in;
model.gene_start_states_out = gene_start_states_out;
model.mRNA_start_states_out = mRNA_start_states_out;
model.gene_stop_states = gene_stop_states;

model.A = A;
model.p = p;
model.q = q;
model.orf_info = orf_info;
model.active_transitions = find(~isinf(A)) ;
model.cnt_transitions = length(model.active_transitions) ;



return

function model = define_transition_pointers(model)
  
A = model.A;
mRNA_start_states_out=model.mRNA_start_states_out;
%%% transition_pointers contain for each allowed transition pointers to lengths plifs
transition_pointers = zeros(size(A)) ;
transition_pointers(~isinf(A))=A(~isinf(A)) ;
transition_pointers = transition_pointers' ;

%%% add pointers to content PLIFs
transition_pointers(:,:,2) = zeros(size(transition_pointers,1), size(transition_pointers,2), 1) ;
for i=1:size(model.plif_links,1)
  [idx1,idx2] = find(transition_pointers(:,:,1)==model.plif_links(i,1));
  for j=1:length(idx1)
    transition_pointers(idx1(j),idx2(j),2) = model.plif_links(i,2);
  end ;
end


%%% add pointers to tiling PLIFs
if model.use.addfeatures.tiling
  dim3 = size(transition_pointers,3);
  transition_pointers(:,:,dim3+1) = zeros(size(transition_pointers,1), size(transition_pointers,2), 1) ;
  for i=1:size(model.tiling_links,1)
    [idx1,idx2] = find(transition_pointers(:,:,1)==model.tiling_links(i,1));
    for j=1:length(idx1)
      transition_pointers(idx1(j),idx2(j),3) = model.tiling_links(i,2);
    end ;
  end
end

%%% add pointers to frame PLIFs
tmp = zeros(size(transition_pointers(:,:,1))) ;
tmp(mRNA_start_states_out, model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% single cds exon  

if ~isfield(model.use.contents, 'pre_comp') || ~model.use.contents.pre_comp,
  tmp(mRNA_start_states_out, model.state_ids.don(2)) = model.contents.cds_frame0 ;% first cds exon 
  tmp(mRNA_start_states_out, model.state_ids.don(3:4)) = model.contents.cds_frame1 ;% first cds exon  
  tmp(mRNA_start_states_out, model.state_ids.don(5:7)) = model.contents.cds_frame2 ;% first cds exon 
  
  tmp(model.state_ids.acc(2:7), model.state_ids.don(2)) = model.contents.cds_frame0 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.don(3:4)) = model.contents.cds_frame1 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.don(5:7)) = model.contents.cds_frame2 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% last cds exon 
else
  tmp(mRNA_start_states_out, model.state_ids.don(2:7)) = model.contents.cds_frame0 ;% first cds exon 
  tmp(model.state_ids.acc(2),model.state_ids.don(2:7)) = model.contents.cds_frame0 ;% middle cds exon 
  tmp(model.state_ids.acc(3:4),model.state_ids.don(2:7)) = model.contents.cds_frame1 ;% middle cds exon 
  tmp(model.state_ids.acc(5:7),model.state_ids.don(2:7)) = model.contents.cds_frame2 ;% middle cds exon 
  tmp(model.state_ids.acc(2), model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% last cds exon 
  tmp(model.state_ids.acc(3:4), model.state_ids.cdsStop(1)) = model.contents.cds_frame1 ;% last cds exon 
  tmp(model.state_ids.acc(5:7), model.state_ids.cdsStop(1)) = model.contents.cds_frame2 ;% last cds exon 
end ;


dim3 = size(transition_pointers,3);
transition_pointers(:,:,dim3+1) = tmp' ; % don't touch this---tmp has to be transposed!

a_trans = zeros(4,sum(~isinf(A(:)))) ;
% [from,to,val,segment]
k = 0 ;
for i=1:size(A,1)
  idx = find(~isinf(A(i,:))) ;
  val = A(i,idx) ;
  pen = transition_pointers(idx,i,1) ;
  a_trans(1,k+1:k+length(idx))=i-1 ;
  a_trans(2,k+1:k+length(idx))=idx-1 ;  
  a_trans(3,k+1:k+length(idx))=val ;    
  for q=1:length(pen) 
    idx_=find(model.seg_links(:,1)==pen(q)) ;
    if isempty(idx_), 
      assert(pen(q)==0) ; 
    else
      assert(length(idx_)==1) ;
      pen(q) = model.seg_links(idx_,2) ;
    end ;
  end ;
  a_trans(4,k+1:k+length(idx))= pen;
  k=k+length(idx) ;
end ;
%a_trans = a_trans' ;
%[tmp,idx] = sort(a_trans(:,2)) ;
%a_trans = a_trans(idx,:) ;

model.transition_pointers = transition_pointers ;
model.a_trans = a_trans;

return

function model = define_loss(model)
  

% create loss matrices

equiv = [];
  
different = [model.segments.intergenic, model.segments.cds_exon, 3 ;
             model.segments.intergenic, model.segments.intron, 3;
             model.segments.intron, model.segments.cds_exon, 2];

%--------------------------------
if ~model.use.loss.orf_only
  different = [different;
               model.segments.intergenic, model.segments.utr5exon, 2;
               model.segments.intergenic, model.segments.utr3exon, 2];
  if model.use.segments.intercistronic
    different = [different;
                 model.segments.intercistronic, model.segments.utr5exon, 2;
                 model.segments.intercistronic, model.segments.utr3exon, 2;
                 model.segments.intergenictrans, model.segments.utr5exon, 2;
                 model.segments.intergenictrans, model.segments.utr3exon, 2];
    
  end
else
  equiv = [equiv;
           model.segments.intergenic model.segments.utr3exon ;
           model.segments.intergenic model.segments.utr5exon] ;
  if model.use.segments.intercistronic
    equiv = [equiv;
             model.segments.intercistronic model.segments.utr3exon ; 
             model.segments.intercistronic model.segments.utr5exon ;
             model.segments.intergenictrans model.segments.utr3exon ;
             model.segments.intergenictrans model.segments.utr5exon] ;
  end
end
%--------------------------------
if model.use.segments.polya_tail
  equiv = [equiv;
           model.segments.utr3exon, model.segments.polya_tail];
  different = [different;
               model.segments.intergenic, model.segments.polya_tail, 1];    
end
 %-------------------------------- 
if model.use.segments.intercistronic
  
  different = [different;
               model.segments.intercistronic, model.segments.cds_exon, 3 ;
               model.segments.intercistronic, model.segments.intron, 3;
               model.segments.intercistronic, model.segments.transexon, 2;
               model.segments.intergenic, model.segments.intercistronic, 0.1;
               model.segments.intergenictrans, model.segments.intercistronic, 0.1;
               model.segments.intergenictrans, model.segments.cds_exon, 3 ;
               model.segments.intergenictrans, model.segments.intron, 3;
               model.segments.intergenictrans, model.segments.transexon, 2;
               model.segments.intergenic, model.segments.intergenictrans, 0.1];

  if model.use.segments.polya_tail
    different = [different;
                 model.segments.intercistronic, model.segments.polya_tail,1;
                 model.segments.intergenictrans, model.segments.polya_tail, 1];
  end
end
%--------------------------------
if model.use.segments.transexon
  equiv = [equiv;
           model.segments.utr5exon, model.segments.transexon] ;     
  different = [different;
               model.segments.intergenic, model.segments.transexon, 1] ;
  if isfield(model.segments, 'intergenictrans'),
    different = [different;
                 model.segments.intergenictrans, model.segments.transexon, 1];    
  end ;
end
%--------------------------------

model.loss.segments = ones(model.cnt_segments) - eye(model.cnt_segments);
for i=1:size(different,1)
  model.loss.segments(different(i,1)+1, different(i,2)+1) = different(i,3) ;
  % model.loss.segments(different(i,1)+1, different(i,2)+1) = different(i,3)+1 ;warning('make loss matrix asymetric to enhance sensitivity')
  model.loss.segments(different(i,2)+1, different(i,1)+1) = different(i,3) ;
end ;

for i=1:size(equiv,1)
  model.loss.segments(equiv(i,1)+1, equiv(i,2)+1) = 0 ;
  model.loss.segments(equiv(i,2)+1, equiv(i,1)+1) = 0 ;
end ;


% tripple loss for coding
%model.loss.segments(model.segments.cds_exon+1, :) = 3*model.loss.segments(model.segments.cds_exon+1, :) ;
%model.loss.segments(:, model.segments.cds_exon+1) = 3*model.loss.segments(:, model.segments.cds_exon+1) ;

% first part is for segments, the second for nucleotides
model.loss.segments = [model.loss.segments 0.0*model.loss.segments] ;

if model.use.signals.tss
  model.loss.zero_range.tss = 10 ;
end
model.loss.zero_range.tis = 0;
model.loss.zero_range.don = 0;
model.loss.zero_range.acc = 0;
if model.use.signals.trans
  model.loss.zero_range.trans = 0;
end
model.loss.zero_range.cdsStop = 0;
if model.use.signals.polya
  model.loss.zero_range.polya = 10 ;
end
if model.use.signals.cleave
  model.loss.zero_range.cleave = 10 ;
end

model.loss.confirmed = 1 ;
model.loss.confirmed_tol = 1000 ; % region around confirmed gene
model.loss.unconfirmed = 1 ;

% model.loss.mustbe_corr.intergenic = 0 ;
% model.loss.mustbe_corr.utr5exon   = 0;
% model.loss.mustbe_corr.transexon  = 0 ;
% model.loss.mustbe_corr.cds_exon   = 1 ;
% model.loss.mustbe_corr.utr3exon   = 0 ;
% model.loss.mustbe_corr.polya_tail = 0 ;
% model.loss.mustbe_corr.intercistronic  = 0 ;
% model.loss.mustbe_corr.intron     = 1 ;
return ;

return

function weights=init_weights(model) ;
% weights=init_weights(model) ;

weights = struct;

%%% content detectors
content_names = fieldnames(model.contents) ;
for s = 1:model.cnt_contents,
  weights = setfield(weights, ['contents_' content_names{s}], zeros(1,model.bins));
end

%%% length detectors
length_names = fieldnames(model.lengths) ;
for s = 1:model.cnt_lengths,
  weights = setfield(weights, ['lengths_' length_names{s}], zeros(1,model.bins));
end

%%% signal detectors
signal_names = fieldnames(model.signals) ;
for s = 1:model.cnt_signals,
  weights = setfield(weights, ['signals_' signal_names{s}], zeros(1,model.bins));
end

%%% tiling detectors
if isfield(model, 'tiling')
  tiling_names = fieldnames(model.tiling) ;
  for s = 1:model.cnt_tiling,
    weights = setfield(weights, ['tiling_' tiling_names{s}], zeros(1,model.bins));
  end
end
%%% rna_seq detectors
if isfield(model, 'rna_seq')
  rna_seq_names = fieldnames(model.rna_seq) ;
  for s = 1:model.cnt_rna_seq,
    weights = setfield(weights, ['rna_seq_' rna_seq_names{s}], zeros(1,model.bins));
  end
end
%%% rna_seq_intron_track detectors
if isfield(model, 'rna_seq_intron_track')
  rna_seq_intron_track_names = fieldnames(model.rna_seq_intron_track) ;
  for s = 1:model.cnt_rna_seq_intron_track,
    weights = setfield(weights, ['rna_seq_intron_track_' rna_seq_intron_track_names{s}], zeros(1,model.bins));
  end
end

%%% transitions
weights.transitions = zeros(1,model.cnt_transitions) ;

return

function [POS,LABEL,CONF] = get_cand_others(P)
% [POS,LABEL,CONF] = get_cand_others(P)

debug = 0;  

genes = P.genes ;
seq = P.seq ;
Signal = P.Signal ;
info = P.info ;
strand = P.strand ;
signal_name = P.signal_name ;
%clear P

%[Signal,info,info_names,sig] = initialize_cand_search(PAR,genes,strand,signal_name,debug);
[Signal,info_names,sig] = initialize_cand_search(Signal, info, genes, strand, signal_name, debug);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% GET ALL CONSENSUS SITES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(Signal.consensus)
  fprintf(1,'getting all consensus sites for: %s\n',Signal.consensus{:})
  POS =  find_consensus(seq,Signal,strand);
else
  fprintf(1,'no consensus given, all positions are potential candidates\n')
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GET ALL POSITIVE SITES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

num = 0;
pos = zeros(1,length(seq));
conf = -ones(length(info_names),length(seq))*200;
gene_idx = zeros(1,length(seq));
for g_idx = 1:length(genes)
  gene = genes(g_idx) ;
  if ~isempty(gene.(signal_name))
    unique_pos = unique([gene.(signal_name){:}]);
    unique_pos2 = setdiff(unique_pos, pos(1:num)) ;
    if ~isequal(unique_pos2, unique_pos),
      warning('identified other genes with same signal positions. Positions ignored')
      unique_pos = unique_pos2 ;
    end ;
    pos(num+[1:length(unique_pos)]) = unique_pos;
    gene_idx(num+[1:length(unique_pos)]) = repmat(g_idx,length(unique_pos),1);
    % === assertion fails for A. thaliana TSS !!! ===
    assert(length(unique(pos(1:num+length(unique_pos))))==length(pos(1:num+length(unique_pos))))
    conf_temp = zeros(length(info_names),length(unique_pos));
    for i=1:length(info_names)
      idx = find(gene.([signal_name '_info'])== ...
                 PAR.info_genes.(signal_name).(info_names{i}));
      pos_temp = [gene.(signal_name){idx}];
      [temp,idx1,idx2] = intersect(unique_pos,pos_temp);
      assert(length(idx) == length(pos_temp))
      conf_temp(i,idx1) = gene.([signal_name '_conf'])(idx(idx2));
    end
    conf(:,num+[1:length(unique_pos)]) = conf_temp; 
    num = num+length(unique_pos);
  end
end; %%%loop over genes
fprintf( 'found %d positive %s signals\n', num, signal_name );
pos(num+1:end) = [];
conf(:,num+1:end) = [];

if isempty(Signal.consensus)
  invalid_true = 0;
  neg_pos = find(~ismember([1:length(seq)],pos));
else
  invalid_true = length(setdiff(pos,POS));
  if invalid_true>10
    warning('%i positions did not exhibit the defined consensus', invalid_true) ;
  end
  pos = intersect(pos,POS); 
  neg_pos = setdiff(POS,pos);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SAMPLE DOWN
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
randn('state',24593854);
rand('state',24593854);

%neg_pos = sort(neg_pos) ;
fprintf(1,'Sampling down negatives:%d\n',Signal.sampling.downsample_neg);
if Signal.sampling.downsample_neg~=1
  var_ = 1/Signal.sampling.downsample_neg*1/5;
  %mean_ = Signal.sampling.mindist+1/Signal.sampling.downsample_neg;
  mean_ = 1/Signal.sampling.downsample_neg;
  num_new_pos = ceil((length(neg_pos)/mean_)*1.1);
  dist = abs(ceil(randn(num_new_pos,1).*var_+ mean_));
  %dist(dist<Signal.sampling.mindist)=dist(dist<Signal.sampling.mindist)+Signal.sampling.mindist;
  dist(dist==0) = deal(Signal.sampling.mindist);
  dd = cumsum(dist);
  neg_pos = intersect(neg_pos,dd);
  %keep = ceil(rand(1,length(neg_pos)*Signal.sampling.downsample_neg)*length(neg_pos));
  %keep = unique(keep);
  %neg_pos = neg_pos(keep);
elseif Signal.sampling.downsample_neg==1 & Signal.sampling.mindist>1
  remove = 1;
  while remove,
    dist = neg_pos(2:end)-neg_pos(1:end-1) ;
    idx = find(dist<Signal.sampling.mindist)+1;
    neg_pos(idx) = [];
    if isempty(idx)
      remove = 0;
    end
  end
end

POS = [pos neg_pos];
LABEL = [ones(1,length(pos)),-ones(1,length(neg_pos))]; 
COV1 = [conf zeros(size(conf,1),length(neg_pos))]; 


[temp,idx] = unique(POS);
assert(length(temp)==length(POS));
POS = POS(idx);
LABEL = LABEL(idx);
COV1 = COV1(:,idx);
if ~isequal(determine_engine, 'octave'), 
  assert(issorted(POS));
end ;


ALL = initialize_allconfs(POS,info_names);
ALL.COV1 = COV1;

  
fprintf('processing: ') ;
num_invalid_genes = 0;
for g_idx = 1:length(genes)
  if mod(g_idx, 100)==0,
    fprintf('%i/%i .. ', num, length(genes));
  end ;
  num = num+1;
  gene = genes(g_idx) ;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % GET BASIC STUFF, 
  % if candidate pos is in gene, get gene id, if gene is valid, if
  % candidate pos is in alternative region 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug,signal_name);
  num_invalid_genes  = num_invalid_genes + is_invalid;
end %%%loop over genes
fprintf('Done\n') ;


if nargout>=3,
  CONF = set_CONF(Signal, ALL, sig, info_names, signal_name);
end ;


% assert(all(sum(Conf(1:3,LABEL==1),1)))
% assert(size(Conf,1)==length(Signal.Conf_names));
fprintf(1,'number of invalid genes: \t\t\t%i\n',num_invalid_genes)
fprintf(1,'number of true %s sites: \t\t\t%i\n',signal_name,sum(LABEL==1))
fprintf(1,'number of invalid true sites (no consensus): \t%i\n',invalid_true)
fprintf(1,'number of decoy sites:\t\t\t\t%i\n',length(LABEL))
% fprintf(1,'mean confidence:\t\t\t\t%.2f\n',mean(sum(Conf,1)));

return

function [Signal,info_names,sig,alt_names] = initialize_cand_search(Signal,info,genes,strand,signal_name,debug)
% [Signal,info,info_names,sig,alt_names] = initialize_cand_search(PAR,genes,strand,signal_name,debug)

rand('seed',1234);

% info = PAR.info_genes.(signal_name);
% Signal = PAR.Signals.(PAR.Signal_name);

if isempty(Signal.consensus) && isfield(Signal, 'consensus_fn') && ~isempty(Signal.consensus_fn)
  load(Signal.consensus_fn,'consensus')
  Signal.consensus = consensus;
end

info_names = fieldnames(info);
use = [];
fprintf(1,'getting conf for ');
for i=1:length(info_names)
  if ismember([info_names{i} '_conf'],Signal.Conf_names)
    use = [use i];
    if isequal(signal_name,'transcript')&~isequal(Signal.name,'transacc')
      assert(ismember([info_names{i} '_conf_skip'],Signal.Conf_names))
    end
    fprintf(1,'%s, ',info_names{i})
  end
end
fprintf('\n')
info_names = info_names(use);

% if ismember('cDNA',info_names)
%   use = strmatch('cDNA',info_names,'exact');
%   info_names{use,2} = 'fulllength';
%   fprintf(1,'\nfulllength cDNAs added to cDNAs\n')
% end

fprintf(1,'\n');

if ~isempty(genes)
  strands = unique([genes.strand]);
  assert(strand==strands(1))
  assert(length(strands)==1);
  assert(length(unique([genes.chr_num]))==1);
end


sig='';
switch Signal.name
 case {'acc','transacc','alt_exon_acc'}
  sig = 'acc';
 case {'don','alt_exon_don'}
  sig = 'don';
 otherwise
  sig = 'other';
end

if isequal(sig,'other') || isequal(Signal.name,'transacc') || ...
      isequal(Signal.name,'acc') || isequal(Signal.name,'don') 
  alt_names ={};
  return
end

alt_names = {'alt_exon', 'alt_intron', 'alt_5prime', 'alt_3prime'};
use = []; 
fprintf(1,'getting label 2 for ');
for i=1:length(alt_names)
  if ismember(alt_names{i},Signal.Conf_names)
    fprintf(1,'%s, ',alt_names{i})
  end
end

return

function ALL = initialize_allconfs(POS,info_names)

ALL.POS = POS;  
  
ALL.GENE_ID = zeros(1,length(POS)); 
ALL.ALTGENIC = zeros(1,length(POS)); 
ALL.COV1 = zeros(length(info_names),length(POS)); 
ALL.COV2 = zeros(length(info_names),length(POS)); 
ALL.LABEL_2 =  -ones(3,length(POS)); 
ALL.ISSPLICE = zeros(1,length(POS)); 

ALL.TRANS = zeros(1,length(POS)); 
ALL.TSS = zeros(1,length(POS)); 
ALL.HAS_SIG = zeros(1,length(POS));
ALL.DIST = zeros(1,length(POS)); 
ALL.CONF_TRANS = zeros(1,length(POS)); 

return

function [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug,signal_name)
% [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug,signal_name)

if nargin<5
  signal_name =[];
end

is_invalid=0; 

genic_idx = find(ALL.POS>gene.start&ALL.POS<gene.stop);
ALL.GENE_ID(genic_idx) = gene.id;

if isfield(gene,'transacc') && ~isempty(gene.transacc)
  ALL.TRANS(genic_idx) = 1;
end
if ~isempty(gene.tss)
  ALL.TSS(genic_idx) = 1;
end

if ~isempty(signal_name)&&~isempty(gene.(signal_name))
  ALL.HAS_SIG(genic_idx) = 1;
end
  
if ~gene.is_valid
  is_invalid=1; 
  ALL.GENE_ID(genic_idx) =  - gene.id;
  return
end

if isequal(sig,'other')
  if gene.is_alt 
    ALL.ALTGENIC(genic_idx) = 1;
  end
else
  if any(gene.exon_in_alt_region)
    alt_reg = get_connected_regions(gene.exon_in_alt_region);  
    
    for i=1:length(unique(alt_reg))-1
      temp=find(alt_reg==i);
      alt_start = gene.splicegraph{1}(1,temp(1));
      alt_stop = gene.splicegraph{1}(2,temp(end));
      alt_genic_idx = find(ALL.POS>alt_start&ALL.POS<alt_stop);
      ALL.ALTGENIC(alt_genic_idx) = 1;
      if debug
        viewsplicegraph(gene,1)
        hold on 
        plot([alt_start alt_start],[1 10],'b')
        plot([alt_stop alt_stop],[1 10],'b')  
      end
    end
  end
end

if gene.is_alt 
  ALL.ALTGENIC(genic_idx) = 1;
end
return

function CONF = set_CONF(Signal,ALL,sig,info_names,signal_name)
% CONF = set_CONF(Signal,ALL,sig,info_names)

%%% GENERAL 
idx = find(strcmp(Signal.Conf_names,'altgenic'));
if ~isempty(idx)
  CONF(idx,:) = ALL.ALTGENIC;
end
idx = find(strcmp(Signal.Conf_names,'gene_id'));
if ~isempty(idx)
  CONF(idx,:) = ALL.GENE_ID;
end

for i=1:length(info_names)
  idx = find(strcmp(Signal.Conf_names,[info_names{i} '_conf']));
  CONF(idx,:) = ALL.COV1(i,:);
  if ~isequal(sig,'other')&~isequal(Signal.name,'transacc')
    idx = find(strcmp(Signal.Conf_names,[info_names{i} '_conf_skip']));
    CONF(idx,:) = ALL.COV2(i,:);
  end
end

%%% ALT SPLICE
idx = find(strcmp(Signal.Conf_names,'alt_exon'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(1,:);
end
idx = find(strcmp(Signal.Conf_names,'alt_intron'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(2,:);
end
idx = find(strcmp(Signal.Conf_names,'alt_3prime'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(3,:);
end
idx = find(strcmp(Signal.Conf_names,'alt_5prime'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(3,:);
end

idx = find(strcmp(Signal.Conf_names,['is' sig]));
if ~isempty(idx)
  CONF(idx,:) = ALL.ISSPLICE;
end

%%% SPECIAL THINGS
idx = find(strcmp(Signal.Conf_names,['has_' signal_name]));
if ~isempty(idx)
  CONF(idx,:) = ALL.HAS_SIG;
end
idx = find(strcmp(Signal.Conf_names,'has_trans'));
if ~isempty(idx)
  CONF(idx,:) = ALL.TRANS;
end


idx = find(strcmp(Signal.Conf_names,'dist'));
if ~isempty(idx)
  CONF(idx,:) = ALL.DIST;
end

if exist('CONF', 'var') && ~(size(CONF,1)==length(Signal.Conf_names))
  warning('this was an assert') ;
end ;

return

function save_predictions_as_sigpred(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)
% save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;

fn_out=[out_fn_prefix '_' score_name '.spf'];

if fexist(fn_out) | fexist([fn_out '.gz'])
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in signal prediction format (spf): %s\n', fn_out);
[fd msg] = fopen(fn_out, 'w+');

fprintf(fd, '##Seqid\tType\tSource\tStart\tStrand\tScore\n') ;
for j=1:length(pos)
  fprintf(fd, '%s\t%s\t%s\t%i\t%c\t%.3f\n', contig_name, signal_name, score_name, pos(j), strand, score(j)) ;
end

fclose(fd);

if do_gzip,
  ret = unix(sprintf('gzip %s',fn_out)) ;
  if ret~=0
    warning('gzip failed') ;
  end
end 


return

function save_predictions_as_sigpred_bin(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand)
% save_predictions_as_sigpred_bin(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

fn_out=[out_fn_prefix '_' score_name '_spf.mat'];

if fexist(fn_out) 
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in binary signal prediction format (spf): %s\n', fn_out);
save(fn_out, '-V7', 'contig_name', 'signal_name', 'score_name', 'pos', 'strand', 'score') ;

return

function fn_out = save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)
% fn_out = save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;

fn_out=[out_fn_prefix '_' score_name '.wiggle'];

if fexist(fn_out)|fexist([fn_out '.gz'])
  fprintf(1, 'file exists, it will NOT be overwritten\n')
%  keyboard
  return
end

fprintf(1,'save as wiggle: %s\n', fn_out);
[fd msg] = fopen(fn_out, 'w+');
disp(msg)

fprintf(fd,'browser position chr%s:1-50,000\n', contig_name);
fprintf(fd,'browser hide all\n');
fprintf(fd,'browser pack Genes\n');
fprintf(fd,'browser full altGraph\n');
fprintf(fd,'browser viewLimits=%1.2f:%1.2f\n', min(score), max(score));


fprintf(fd,['track type=wiggle_0 name="%s_%s%s" description="%s chr%s %s" ' ...
            '\n'],signal_name, score_name, strand, score_name, ...
        contig_name, strand);
fprintf(fd,'variableStep chrom=chr%s\n',contig_name);

for j=1:length(pos)
  fprintf(fd,'%i\t%.3f\n', pos(j), score(j));
end

fclose(fd);

if do_gzip,
  ret = unix(sprintf('gzip %s',fn_out));
  if ret~=0
    warning('gzip failed')
  end
end ;

return

function POS=  find_consensus(seq,Signal,strand)
%  POS=  find_consensus(seq,Signal,strand) 

consensus = Signal.consensus ; 
lwin_big = Signal.lwin_big;
rwin_big = Signal.rwin_big;
offset = Signal.offset;



POS = zeros(1,floor(length(seq)/10));
num = 0;
seq=upper(seq);
for m = 1:length(consensus)  
  cons = upper(consensus{m});
  pos_tmp = findstr(seq(lwin_big+1:end-rwin_big),cons)+lwin_big;
  POS(num+[1:length(pos_tmp)]) = pos_tmp;
  num = num + length(pos_tmp);
end
POS(num+1:end)=[];
clear num 
POS = POS + offset;
%if strand=='-' & strcmp(Signal.name,'polya')
%  POS = -POS + length(seq)+1-length(cons)+1;
%elseif strand=='-' 
if strand=='-'
  POS = -POS + length(seq)+1;
end
POS = unique(sort(POS));

return

function [POS,LABEL,CONF] = get_cand_splicesites(P)
  
% [pos,label,Conf] = get_cand_splicesites(genes,seq,PAR)
% WARNING IMPLENENTATION OF COV_SKIPPED NOT FINISHED!!!

debug = 0;  
  
genes = P.genes ;
seq = P.seq ;
Signal = P.Signal ;
info = P.info ;
strand = P.strand ;
%clear P
  
% info = PAR.info_genes.(signal_name);
[Signal,info_names,sig,alt_names] = initialize_cand_search(Signal,info,genes,strand,'transcript',debug);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% GET ALL CONSENSUS SITES / INITIALIZE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

POS = find_consensus(seq,Signal,strand);

ALL = initialize_allconfs(POS, info_names);
ALL.LABEL =  -ones(1,length(POS)); 
clear POS;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% GET TRANSSPLICE SPECIFIC STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isequal(Signal.name,'transacc')
  ALL = get_cand_transacc(genes,ALL,Signal,info_tran);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% detecting exons in regions of alternative splicing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('detecting exons in regions of alternative splicing') ;
for i=1:length(genes)
  genes(i).alt_regions = detectaltregions(genes(i)) ;
  map = zeros(1,size(genes(i).splicegraph{1},2)) ;
  for j=1:size(genes(i).alt_regions,2),
    map = map | segment_overlap(genes(i).alt_regions(1,j)+1, ...
                                genes(i).alt_regions(2,j)-1, ...
                                genes(i).splicegraph{1}(1,:), ...
                                genes(i).splicegraph{1}(2,:)) ;
  end ;
  if ~isempty(genes(i).alt_regions),
    assert(any(map==1)) ;
  end ;
  
  %extent alt_region to neigbouring exons
  map = [0 map 0];
  idx_temp = find(map);
  map(idx_temp+1)=1;
  map(idx_temp-1)=1;
  genes(i).exon_in_alt_region=map(2:end-1) ;
  %% Version 4225 works without extension
end ;

genes = annotate_splicegraph(genes);
% genes = cover_splicegraph(genes);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% START LOOP OVER GENES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

num_invalid_genes = 0;
invalid_true = 0;

for g_idx = 1:length(genes)
  
  
  %fprintf(1,'%i\r',g_idx);
  gene = genes(g_idx) ;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % GET BASIC STUFF, 
  % if candidate pos is in gene, get gene id, if gene is valid, if
  % candidate pos is in alternative region 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug);
  num_invalid_genes  = num_invalid_genes + is_invalid;
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  % GET TRUE SPLICE SITES
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  [don_exons, acc_exons] = ind2sub(size(gene.splicegraph{2}), find(triu(gene.splicegraph{2}))) ;
  true_splice = [];
  
  if (isequal(sig,'acc') && strand=='+')||...
        (isequal(sig,'don') && strand=='-'),
    true_splice = unique(gene.splicegraph{1}(1,acc_exons)) ;
  elseif (isequal(sig,'don')&& strand=='+') || ...
        (isequal(sig,'acc')&& strand=='-')
    true_splice = unique(gene.splicegraph{1}(2,don_exons)) ;
  end
   
  [tmp,ii1,ii2] = intersect(ALL.POS,true_splice);
  
  if length(tmp)<length(true_splice)
    invalid_true = invalid_true+length(true_splice)-length(tmp) ;
  end
  if ~isequal(Signal.name,'transacc')
    ALL.LABEL(ii1) = 1;
  else
    ALL.ISSPLICE(ii1) = 1;
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  % GET TRUE ALTERNATIVE SPLICE SITES
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

  if ~isempty(alt_names)
    [ALL,label_2] = get_true_alternative_sites(ALL,gene,true_splice,sig,debug);
  else
    label_2 = -ones(3,length(true_splice)); 
  end 
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  %%% CHECK COVERAGE 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  
  if ~isempty(info_names) 
    ALL = get_coverage(ALL, gene, true_splice, label_2, info, info_names);
  end
      
  % keyboard
end %%%loop over genes

if isequal(Signal.name,'transacc')
  ALL.COV1 = ALL.CONF_TRANS;
end
if nargout>=3,
  CONF = set_CONF(Signal, ALL, sig, info_names, '');
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SAMPLE DOWN
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pos=ALL.POS(ALL.LABEL==1);
neg_pos=ALL.POS(ALL.LABEL==-1);

% pos contains positive examples, neg_pos contains negative examples

%if Signal.sampling.downsample_neg~=1
%  fprintf(1,'Sampling down negatives:%d',Signal.sampling.downsample_neg);
%  randn('state',24593854);
%  dist = abs(ceil(randn(length(neg_pos),1)./Signal.sampling.downsample_neg/5+ Signal.sampling.mindist+1/Signal.sampling.downsample_neg));
%  dist(dist<Signal.sampling.mindist)=dist(dist<Signal.sampling.mindist)+Signal.sampling.mindist;
%  dd = cumsum(dist);
%  neg_pos = intersect(neg_pos,dd);
%elseif Signal.sampling.downsample_neg==1 & Signal.sampling.mindist>1
%  remove = 1;
%  while remove
%    dist = neg_pos(2:end)-neg_pos(1:end-1)
%    idx = find(dist<Signal.sampling.mindist)+1;
%    neg_pos(idx) = [];
%    if isempty(idx)
%      remove = 0;
%    end
%  end
%end

POS=[pos neg_pos];
LABEL=[ones(1,length(pos)), -ones(1,length(neg_pos))];

% keyboard
% assert(all(sum(Conf(1:3,LABEL==1),1)))
% assert(size(Conf,1)==length(Signal.Conf_names));
fprintf(1,'number of invalid genes: \t\t\t%i\n',num_invalid_genes)
fprintf(1,'number of true %s sites: \t\t\t%i\n',Signal.name,sum(LABEL==1))
fprintf(1,'number of invalid true sites (no consensus): \t%i\n',invalid_true)
fprintf(1,'number of decoy sites:\t\t\t\t%i\n',length(LABEL))
% fprintf(1,'mean confidence:\t\t\t\t%.2f\n',mean(sum(Conf,1)));

if invalid_true>0
  % keyboard
end

return

function [regions,level] = detectaltregions(gene)
% regions = detectaltregions(gene)
  
[max_exon_level,max_intron_level,exon_level,intron_level,level] = ...
    detectsplicegraph(gene) ;

if all(level(3,:)<=1), 
  regions=[] ;
  return ;
else
  regions = [] ;
  idx=find(level(3,:)>1) ;
  idx=unique([idx(1) find(level(3,2:end)>1 & level(3,1:end-1)<=1)+1]);

  
  for i=1:length(idx)
    %level(4,idx(i))
    %if level(4,idx(i))==0, keyboard; end ;

  %    if idx(i)==1
      start = level(1,idx(i)) ;
%    else
%      start = floor(mean([level(1,idx(i)-1) level(1,idx(i))])) ;
%    end ;
%    if idx(i)==size(level,2)
      stop = level(2,idx(i)) ;
%    else
%      stop = floor(mean([level(2,idx(i)) level(2,idx(i)+1)]))-1 ;
%    end ;
    depth = level(3,idx(i)) ;
    for j=idx(i):size(level,2),
      if level(3,j)>1 
%        if j==size(level,2)
          stop = level(2,j) ;
%        else
%          stop = floor(mean([level(2,j) level(2,j+1)]))-1 ;
%        end ;
        depth = max(depth, level(3,j)) ;
      else
        %if level(4,j)==0, keyboard; end ;
        break ;
      end ;
    end ;
    regions=[regions, [start; stop; depth]] ;
  end ;
end ;
return

function [max_exon_level,max_intron_level,exon_level,intron_level,level] = detectsplicegraph(gene)

  [max_exon_level,max_intron_level,exon_level,intron_level,level] = countlevel(gene.exons,gene.name,...
                                                    gene.transcripts,[], ...
                                                    gene.splicegraph{1},...
                                                    gene.splicegraph{2});
  return


function [max_level,max_leveli,exon_level,intron_level,level] = countlevel(truepos,name,labels,colors,...
                                                    vertices,edges)
  
  
  start=inf ;
  stop=-inf ;
  for k=1:length(truepos)
    start=min(start,truepos{k}(1,1)-50) ;
    stop=max(stop,truepos{k}(end,2)+50) ;
  end ;
  
  
  max_level = 0;
  max_leveli = 0;

  splice = unique(vertices(:)) ;
  
  exon_loc = zeros(1,stop-start);
  exon_level = zeros(3,size(vertices,2));
  for i=1:size(vertices,2)
    cur_vertex = vertices(:,i) - [start;start];
    exon_loc(cur_vertex(1):cur_vertex(2)-1) = ...
        exon_loc(cur_vertex(1):cur_vertex(2)-1) + 1;
    %exon_loc(cur_vertex(1):cur_vertex(2)-1) = ...
    %    max(exon_loc(cur_vertex(1):cur_vertex(2)-1));
    
    level = max(exon_loc(cur_vertex(1):cur_vertex(2)-1));
    if isempty(level), level=0 ; end ;
    max_level = max(level,max_level);
    exon_level(1,i) = vertices(1,i) ;  
    exon_level(2,i) = vertices(2,i) ;  
    exon_level(3,i) = level;  
  end
  
  intron_loc = zeros(1,stop-start);
  intron_level = zeros(3,0) ;
  if (size(edges,1) > 1)
    for i=1:size(vertices,2)
      for j=i+1:size(vertices,2)
        if edges(i,j)
          if (vertices(1,i) < vertices(1,j))
            istart = vertices(2,i);
            istop =  vertices(1,j);
            %level1 = exon_level(i);
            %level2 = exon_level(j);
          else
            istart = vertices(2,j);
            istop =  vertices(1,i);
            %level1 = exon_level(j);
            %level2 = exon_level(i);
          end
          
          cur_intron = [istart-start,istop-start];
          intron_loc(cur_intron(1):cur_intron(2)-1) = ...
              intron_loc(cur_intron(1):cur_intron(2)-1) + 1;
          intron_level(3,end+1) = max(intron_loc(cur_intron(1):cur_intron(2)-1));
          intron_level(1,end) = istart ;
          intron_level(2,end) = istop ;

          max_leveli = max(intron_level(3,end),max_leveli);
        end
      end
    end
  end
  
  splice_loc = exon_loc+intron_loc ;
  level=zeros(5,length(splice)-1) ;
  for i=1:length(splice)-1,
    level(:,i)=[splice(i);
                splice(i+1);
                max(splice_loc([splice(i)-start:splice(i+1)-start-1]));
                max(exon_loc([splice(i)-start:splice(i+1)-start-1]));
                max(intron_loc([splice(i)-start:splice(i+1)-start-1]));
               ] ;
  end ;

return


return

function genes = annotate_splicegraph(genes)
% function annotate_splicegraph(info,origpath)
%
% annotate the genes structure with counts for:
% - exon skipping
% - intron retention
% - alternative donor (5 prime)
% - alternative acceptor (3 prime)
% - alternative transcription start
% - alternative transcription end
%
% Written by: Cheng Soon Ong, 14 May 2007
% Modified on: 31 May 2007 to include transcription starts and stops


%addpath(sprintf('%s/utils', origpath)) ;
%addpath(sprintf('%s/splicegraphs', origpath)) ;
%addpath(sprintf('%s/splicegraphs/detect_altsplice', origpath)) ;
%addpath(sprintf('%s/gff', origpath)) ;
%addpath(sprintf('%s/gff/read_sequences', origpath)) ;
%addpath /fml/ag-raetsch/share/software/matlab_tools/utils

%genome_info = init_genome(info) ;
%fprintf('directory: %s\n', genome_info.basedir) ;
%load(sprintf('%s/confirmed_sequences.mat', genome_info.basedir), 'genes') ;
%fprintf('loaded confirmed_sequences.mat\n\n') ;

if ~isfield(genes, 'strands'),
  for i=1:length(genes),
    genes(i).strands = repmat(genes(i).strand, 1, length(genes(i).transcripts)) ;
  end ;
end ;

if isfield(genes,'alt_3prime_details') || isfield(genes,'alt_5prime_details')
  warning('annotate_splicegraph adds to alt_3prime_details and alt_5prime_details,\nrunning more than one creates inconsistencies');
  return;
end

idx_alt=[] ;
for i=1:length(genes)
  if genes(i).is_alt_spliced
    idx_alt=[idx_alt, i] ;
  end
  if ~isfield(genes(i),'strands') || isempty(genes(i).strands)
    genes(i).strands(1) = genes(i).strand;
  end
end

[idx_exon_skips, exon_exon_skips] = detect_exonskips(genes, idx_alt);
[idx_alt_5prime,exon_alt_5prime, idx_alt_3prime,exon_alt_3prime] = ...
    detect_altprime_pair(genes, idx_alt);
[idx_intron_reten,intron_intron_reten] = detect_intronreten(genes, idx_alt) ;
[idx_xor_exons, exon_xor_exons] = detect_xorexons(genes, idx_alt) ;
[idx_alt_tstart, exon_alt_tstart] = detect_alttstart(genes) ;
idx_alt_tstart = [idx_alt_tstart{:}];
[idx_alt_tend, exon_alt_tend] = detect_alttend(genes) ;
idx_alt_tend = [idx_alt_tend{:}];

%keyboard
for ix = 1:length(genes)
    
  if mod(ix,1000)==0
    fprintf('.');
  end

  exons_is_alt = zeros(1,size(genes(ix).splicegraph{1},2));

  % exon skips
  if ismember(ix,idx_exon_skips)
    tix = find(idx_exon_skips==ix);
    genes(ix).alt_exon = length(tix);
    genes(ix).alt_exon_details.exons = exon_exon_skips(:,tix);
    exons_is_alt(exon_exon_skips(:,tix)) = exons_is_alt(exon_exon_skips(:,tix)) + 1;
  else
    genes(ix).alt_exon = 0;
  end

  % intron retentions
  if ismember(ix,idx_intron_reten)
    tix = find(idx_intron_reten==ix);
    genes(ix).alt_intron = length(tix);
    genes(ix).alt_intron_details.exons = intron_intron_reten(:,tix);
    exons_is_alt(intron_intron_reten(:,tix)) = exons_is_alt(intron_intron_reten(:,tix)) + 1;
  else
    genes(ix).alt_intron = 0;
  end

  % alternative 5prime
  if ismember(ix,idx_alt_5prime)
    genes(ix).alt_5prime_details.exons = zeros(3,0);
    tix = find(idx_alt_5prime==ix);
    genes(ix).alt_5prime = length(tix);
    for ixt = tix
      if genes(ix).strands(1) == '+'
	idx_exon_l = exon_alt_5prime(ixt).fiveprimesites(1);
	idx_exon_r = exon_alt_5prime(ixt).threeprimesite;
	idx_exon_alt = exon_alt_5prime(ixt).fiveprimesites(2);
      else
	idx_exon_r = exon_alt_5prime(ixt).fiveprimesites(1);
	idx_exon_l = exon_alt_5prime(ixt).threeprimesite;
	idx_exon_alt = exon_alt_5prime(ixt).fiveprimesites(2);
      end
      genes(ix).alt_5prime_details.exons(:,end+1) = [idx_exon_l;idx_exon_r;idx_exon_alt];
      exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) = exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) + 1;
    end
  else
    genes(ix).alt_5prime = 0;
  end

  % alternative 3prime
  if ismember(ix,idx_alt_3prime)
    genes(ix).alt_3prime_details.exons = zeros(3,0);
    tix = find(idx_alt_3prime==ix);
    genes(ix).alt_3prime = length(tix);
    for ixt = tix
      if genes(ix).strands(1) == '+'
	idx_exon_l = exon_alt_3prime(ixt).fiveprimesite;
	idx_exon_r = exon_alt_3prime(ixt).threeprimesites(1);
	idx_exon_alt = exon_alt_3prime(ixt).threeprimesites(2);
      else
	idx_exon_r = exon_alt_3prime(ixt).fiveprimesite;
	idx_exon_l = exon_alt_3prime(ixt).threeprimesites(1);
	idx_exon_alt = exon_alt_3prime(ixt).threeprimesites(2);
      end
      genes(ix).alt_3prime_details.exons(:,end+1) = [idx_exon_l;idx_exon_r;idx_exon_alt];
      exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) = exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) + 1;
    end
  else
    genes(ix).alt_3prime = 0;
  end

  % xor exons
  if ismember(ix,idx_xor_exons)
    tix = find(idx_xor_exons==ix);
    genes(ix).alt_xor_exon = length(tix);
    for ixt = tix
      exons_is_alt(exon_xor_exons(:,tix)) = exons_is_alt(exon_xor_exons(:,tix)) + 1;
    end
  else
    genes(ix).alt_xor_exon = 0;
  end

  % alternative transcriptions
  if ismember(ix,idx_alt_tstart)
    tix = find([idx_alt_tstart == ix]);
    % there is only one entry per gene.
    assert(all(size(tix) == [1,1]))
    genes(ix).alt_tstart = length(exon_alt_tstart{tix}.exons);
    genes(ix).alt_tstart_details.exons = exon_alt_tstart{tix}.exons;
  else
    genes(ix).alt_tstart = 0;
  end

  if ismember(ix,idx_alt_tend)
    tix = find([idx_alt_tend == ix]);
    % there is only one entry per gene.
    assert(all(size(tix) == [1,1]))
    genes(ix).alt_tend = length(exon_alt_tend{tix}.exons);
    genes(ix).alt_tend_details.exons = exon_alt_tend{tix}.exons;
  else
    genes(ix).alt_tend = 0;
  end

  genes(ix).exons_is_alt = exons_is_alt;
end


if 0,
  fid = fopen(sprintf('%s/alt_splice_type.tsv', genome_info.basedir),'w');
  fprintf(fid,'Clust\tExon Skip\tIntron Reten\tAlt 5prime\tAlt 3prime\n');
  for ix = idx_alt  
    fprintf(fid,'%d\t%d\t%d\t%d\t%d\n',ix,...
	    genes(ix).alt_exon,genes(ix).alt_intron,genes(ix).alt_5prime,genes(ix).alt_3prime);
  end
  fclose(fid);
end

return

function [idx_exon_skips, exon_exon_skips] = detect_exonskips(genes, idx_alt) ;
% [idx_exon_skips, exon_exon_skips] = detect_exonskips(genes, idx_alt) ;

idx_exon_skips = [];
exon_exon_skips = [];
for ix=idx_alt
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2);
  edges = genes(ix).splicegraph{2};
  for exon_idx = 1:num_exons-2
    for exon_idx1 = (exon_idx+1):(num_exons-1)
      for exon_idx2 = (exon_idx1+1):num_exons
        if (edges(exon_idx,exon_idx1))&&...
              (edges(exon_idx,exon_idx2))&&...
              (edges(exon_idx1,exon_idx2))
          idx_exon_skips = [idx_exon_skips,ix];
          exon_exon_skips = [exon_exon_skips,[exon_idx; exon_idx1; exon_idx2]];
        end
      end
    end
  end
end
sum_skips=length(idx_exon_skips) ;
fprintf(1,'\n\nNumber of single exon skips:\t\t\t\t\t%d\n', sum_skips);



return

function [idx_alt_5prime,exon_alt_5prime, idx_alt_3prime,exon_alt_3prime] ...
    = detect_altprime_pair(genes, idx_alt);
% function [idx_alt_5prime,exon_alt_5prime, idx_alt_3prime,exon_alt_3prime] ...
%    = detect_altprime_pair(genes, idx_alt);
%
% Detect the alternative 5 and 3 prime ends of the intron. Note that 5 prime refers to the left
% and 3 prime to the right for a positive strand
% The difference between this function and detect_altprime.m is that the other function collects
% all the alternative sites corresponding to an acceptor or donor.
% note that the 'overlap' relationship is not transitive

MIN_OVERLAP = 11; % two exons that are alternative should overlap by at least the BLAT seed

idx_alt_5prime = [];
idx_alt_3prime = [];
exon_alt_5prime = {};
exon_alt_3prime = {};

if ~isfield(genes, 'strands'),
  for i=1:length(genes),
    genes(i).strands = repmat(genes(i).strand, 1, length(genes(i).transcripts)) ;
  end ;
end ;

for ix=idx_alt
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  vertices = genes(ix).splicegraph{1} ;
  edges = genes(ix).splicegraph{2} ;
  
  % Find alternative sites on the right of the intron,
  % same site on the left of the intron.
  for exon_idx = 1:num_exons-2
    nr_exons=sum(edges(exon_idx,exon_idx+1:num_exons));
    if nr_exons>=2,
      which_exons = find([zeros(1,exon_idx),edges(exon_idx,exon_idx+1:num_exons)]);
      exons=[vertices(:, which_exons)] ;
      for i=1:nr_exons-1
	for j=i+1:nr_exons
	  % check that the left splice site of the exons are different
	  % make sure exons overlap - either:
	  % - left splice site of exon(i) is in exon(j)
	  % - left splice site of exon(j) is in exon(i)
	  
	  if ((exons(1,i) ~= exons(1,j)) && ...
	       (( (exons(1,i) >= exons(1,j)) && (exons(1,i) <= exons(2,j)) ) ...
		|| ( (exons(1,j) >= exons(1,i)) && (exons(1,j) <= exons(2,i)) )) && ...
	       (min(exons(2,i),exons(2,j))-max(exons(1,i),exons(1,j)) >= MIN_OVERLAP ) )
	    assert(~((exons(1,i) == exons(1,j)) && (exons(2,i) == exons(2,j))));
	    assert(exons(1,i) ~= exons(1,j));
	    % construct the output
	    if (genes(ix).strands(1)=='+'),
	      exon_alt_3prime(end+1).fiveprimesite=exon_idx ;
	      if exons(1,i) > exons(1,j)
		exon_alt_3prime(end).threeprimesites=which_exons([i j]);
	      else
		exon_alt_3prime(end).threeprimesites=which_exons([j i]);
	      end
	      idx_alt_3prime(end+1) = ix;
	    end
	    if (genes(ix).strands(1)=='-'),
	      exon_alt_5prime(end+1).threeprimesite=exon_idx ;
	      if exons(1,i) > exons(1,j)
		exon_alt_5prime(end).fiveprimesites=which_exons([i j]);
	      else
		exon_alt_5prime(end).fiveprimesites=which_exons([j i]);
	      end
	      idx_alt_5prime(end+1) = ix;
	    end
	  end % if altsplice
	  
	end % for j=i+1:nr_exons
      end % i=1:nr_exons-1
    end
  end % for exon_idx

  
  % Find alternative sites on the left of the intron,
  % same site on the right of the intron.
  for exon_idx = 3:num_exons
    nr_exons=sum(edges(1:exon_idx-1,exon_idx)) ;
    if nr_exons>=2,
      which_exons = find([edges(1:exon_idx-1,exon_idx)', zeros(1,num_exons-exon_idx+1)]);
      exons=[vertices(:, which_exons)] ;
      for i=1:nr_exons-1
	for j=i+1:nr_exons
	  % check that the 5prime sites are different
	  % make sure exons overlap - either:
	  % - right splice site of exon(i) is in exon(j)
	  % - right splice site of exon(j) is in exon(i)
	  if ((exons(2,i) ~= exons(2,j)) &&...
	      (( (exons(2,i) <= exons(2,j)) && (exons(2,i) >= exons(1,j)) )...
	      || ( (exons(2,j) <= exons(2,i)) && (exons(2,j) >= exons(1,i)) )) && ...
	      (min(exons(2,i),exons(2,j))-max(exons(1,i),exons(1,j)) >= MIN_OVERLAP ) )
	    assert(~((exons(1,i) == exons(1,j)) && (exons(2,i) == exons(2,j))));
	    assert(exons(2,i) ~= exons(2,j));

	    % construct the output
	    if (genes(ix).strands(1)=='+'),
	      exon_alt_5prime(end+1).threeprimesite=exon_idx ;
	      if exons(2,i) < exons(2,j)
		exon_alt_5prime(end).fiveprimesites=which_exons([i j]);
	      else
		exon_alt_5prime(end).fiveprimesites=which_exons([j i]);
	      end
	      idx_alt_5prime(end+1) = ix;
	    end
	    if (genes(ix).strands(1)=='-'),
	      exon_alt_3prime(end+1).fiveprimesite=exon_idx ;
	      if exons(2,i) < exons(2,j)
		exon_alt_3prime(end).threeprimesites=which_exons([i j]);
	      else
		exon_alt_3prime(end).threeprimesites=which_exons([j i]);
	      end
	      idx_alt_3prime(end+1) = ix;
	    end
	  end % if altsplice
	  
	end % for j=i+1:nr_exons
      end % for i=1:nr_exons-1
    end

  end % construct output
end % for exon_idx
  



fprintf(1,'\n\nNumber of alternative 5 prime sites:\t\t\t\t%d\n',...
	length(idx_alt_5prime));

fprintf(1,'Number of alternative 3 prime sites:\t\t\t\t%d\n',...
	length(idx_alt_3prime));






return

function [idx_xor_exons, exon_xor_exons,id_xor_exons] = detect_xorexons(genes, idx_alt);

idx_xor_exons = zeros(1,0); 
id_xor_exons = zeros(1,0); 
exon_xor_exons = zeros(4,0); %%%% 5primesite of first exon, the 2 skipped
                     %exons, 3primesite of last exon %%%
id = 0 ;
for ix=idx_alt
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2);
  edges = genes(ix).splicegraph{2};
  vertices = genes(ix).splicegraph{1};
  for exon_idx1 = 1:num_exons-3
    id=id+1 ;
    for exon_idx2 = (exon_idx1+1):(num_exons-2)
      if edges(exon_idx1, exon_idx2)
        for exon_idx3=(exon_idx2+1):(num_exons-1)
          if edges(exon_idx1, exon_idx3) & edges(exon_idx2, exon_idx3)==0 & ...
             (vertices(1,exon_idx3) > vertices(2,exon_idx2)),
            for exon_idx4=(exon_idx3+1):num_exons
              if edges(exon_idx2, exon_idx4) & edges(exon_idx3, exon_idx4) & ...
                    edges(exon_idx1, exon_idx4)==0
                idx_xor_exons = [idx_xor_exons, ix];
                id_xor_exons = [id_xor_exons, id];
                exon_xor_exons = [exon_xor_exons,[exon_idx1; exon_idx2; exon_idx3; exon_idx4]] ;
              end
            end
          end
        end
      end
    end
  end
end
fprintf(1,'\n\nNumber of XOR exons:\t\t\t\t\t\t%d\n',...
	length(idx_xor_exons));



return

function [idx_intron_reten,intron_intron_reten] = detect_intronreten(genes,idx_alt) ;

idx_intron_reten = [];
intron_intron_reten = [];
for ix=idx_alt
  %viewsplicegraph(genes(ix)) ;
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2);
  vertices = genes(ix).splicegraph{1};
  edges = genes(ix).splicegraph{2};
  introns=[] ;
  for exon_idx = 1:num_exons-1 %start of intron
    %exon_idx
    %edges(exon_idx,exon_idx+1:length(edges))
    idx=find(edges(exon_idx,exon_idx+1:length(edges))==1) ;
    if isempty(idx), continue; end
    idx=idx+exon_idx ;
    for exon_idx2=idx, %end of intron
      is_intron_reten=0 ;
      for exon_idx1=1:num_exons %exon
	% check that the exon covers the intron
        if (vertices(2,exon_idx) >= vertices(1,exon_idx1)) && (vertices(1,exon_idx2) <= vertices(2,exon_idx1))
	
	% check that the exon covers the intron, but doesn't extend beyond
        %if (vertices(2,exon_idx) > vertices(1,exon_idx1)) && (vertices(1,exon_idx2) < vertices(2,exon_idx1))...
	%      && (vertices(1,exon_idx) <= vertices(1,exon_idx1)) && (vertices(2,exon_idx2) >= vertices(2,exon_idx1))
          is_intron_reten=1 ;
          long_exon=exon_idx1 ;
          for len=1:size(introns,2),
            if (vertices(2,exon_idx)==introns(1,len)) && (vertices(1,exon_idx2)==introns(2,len))
              is_intron_reten=0 ;
            end  
          end
        end
      end
      if is_intron_reten==1,
        idx_intron_reten = [idx_intron_reten, ix];
        intron_intron_reten = [intron_intron_reten, [exon_idx ; exon_idx2 ...
                   ; long_exon]] ;
        introns=[introns, [vertices(2,exon_idx) ; vertices(1,exon_idx2)]] ;
      end
    end
    %pause ;
  end
  
  %pause ;
end
fprintf(1,'\n\nNumber of intron retentions:\t\t\t\t\t%d\n', ...
	length(idx_intron_reten));


return

function [idx_alt_tstart, exon_alt_tstart] = detect_alttstart(genes) ;

idx_alt_tstart = {};
exon_alt_tstart = {};

for ix=1:length(genes)
  if (mod(ix,1000)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  vertices = genes(ix).splicegraph{1} ;
  edges = triu(genes(ix).splicegraph{2}) ;
  
  init_alt_idx = [] ;
  for i=1:num_exons,
    if sum(edges(:,i),1)==0, % initial
      init_alt_idx(end+1) = i ;
    end ;
  end ;

  if length(init_alt_idx)>1,
    idx_alt_tstart{end+1} = ix;
    exon_alt_tstart{end+1}.exons = init_alt_idx;
  end
end

fprintf(1,'\nTotal alternative transcription starts:\t\t\t\t%d\n',...
        length(idx_alt_tstart));

return

function [idx_alt_tend, exon_alt_tend] = detect_alttend(genes) ;

idx_alt_tend = {};
exon_alt_tend = {};

for ix=1:length(genes)
  if (mod(ix,1000)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  vertices = genes(ix).splicegraph{1} ;
  edges = triu(genes(ix).splicegraph{2}) ;
  
  term_alt_idx = [] ;
  for i=1:num_exons,
    if sum(edges(i,:),2)==0,
      term_alt_idx(end+1) = i ;
    end ;
  end ;
  if length(term_alt_idx)>1
    idx_alt_tend{end+1} = ix;
    exon_alt_tend{end+1}.exons = term_alt_idx;
  end
end

fprintf(1,'\nTotal alternative transcription ends:\t\t\t\t%d\n',...
        length(idx_alt_tend));

return

function  [ALL,label_2] = get_true_alternative_sites(ALL,gene,true_splice,sig,debug)

strand = gene.strand;
POS=ALL.POS;
LABEL_2=ALL.LABEL_2;

if (isequal(sig,'acc') & strand=='+')|...
      (isequal(sig,'don') & strand=='-'),
  which_end = 1;
  other_end = 2;
elseif (isequal(sig,'don')& strand=='+') | ...
      (isequal(sig,'acc')& strand=='-')
  which_end = 2;
  other_end = 1;
end
   
  
label_2 = -ones(3,length(true_splice)); 
% GET TRUE ALTERNATIVE SPLICE SITES

ii_es2 = []; ii_o2 = [];  ii_ir2 = [];  ii_3p2 = [];  ii_5p2 = []; 

if gene.alt_exon %% EXON SKIP
  alt_exons = gene.alt_exon_details.exons;
  true_alt_es = gene.splicegraph{1}(which_end,alt_exons(2,:)) ;
  [temp,ii1,ii_es2] = intersect(true_alt_es,true_splice);
  assert(length(temp)==length(unique(true_alt_es)))
  true_alt_es = true_alt_es(ii1);
  % splice_before_es = gene.splicegraph{1}(don_end,alt_exons(1,ii1)) ;
  label_2(1,ii_es2) = 1;
  if debug
    fprintf(1,'exon skip')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_es)
      plot([true_alt_es(jj) true_alt_es(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard
    end ;
  end
end %if gene.alt_exon

if gene.alt_intron
  alt_introns = gene.alt_intron_details.exons;
  true_alt_ir = gene.splicegraph{1}(which_end,alt_introns(other_end,:));
  [temp,ii1,ii_ir2] = intersect(true_alt_ir,true_splice);
  assert(length(temp)==length(unique(true_alt_ir)))
  true_alt_ir = true_alt_ir(ii1);
  % splice_before_ir = gene.splicegraph{1}(acc_end,alt_introns(1,ii1));
  % intron_ret(1,:) = gene.splicegraph{1}(which_end,alt_introns(other_end,ii1));
  % intron_ret(2,:) = gene.splicegraph{1}(other_end,alt_introns(which_end,ii1));
  label_2(2,ii_ir2) = 1;
  if debug
    fprintf(1,'intron retention')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_ir)
      plot([true_alt_ir(jj) true_alt_ir(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard
    end 
  end
end

if gene.alt_5prime&isequal(sig, 'don') 
  %if ~isempty(gene.alt_5prime_details), keyboard; end ;
  if gene.strand=='+',
    alt_5p = gene.alt_5prime_details.exons(1,:);
  else
    alt_5p = gene.alt_5prime_details.exons(2,:);
  end ;
  alt_5p = unique([alt_5p gene.alt_5prime_details.exons(3,:)]);
  true_alt_5p = gene.splicegraph{1}(which_end,alt_5p);
  [temp,ii1,ii_5p2] = intersect(true_alt_5p,true_splice);
  assert(length(temp)==length(unique(true_alt_5p)))
  true_alt_5p = true_alt_5p(ii1);
  % splice_before_5p = gene.splicegraph{1}(acc_end,);
  % don_after = gene.splicegraph{1}(other_end,alt_3p);
  label_2(3,ii_5p2) = 1;
  if debug
    fprintf(1,'alt 5prime')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_5p)
      plot([true_alt_5p(jj) true_alt_5p(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard
    end 
  end
end
if gene.alt_3prime&isequal(sig,'acc')
  %if ~isempty(gene.alt_3prime_details), keyboard; end ;
  if gene.strand=='+',
    alt_3p = gene.alt_3prime_details.exons(2,:);
  else
    alt_3p = gene.alt_3prime_details.exons(1,:);
  end ;
  alt_3p = unique([alt_3p gene.alt_3prime_details.exons(3,:)]);
  true_alt_3p = gene.splicegraph{1}(which_end,alt_3p);
  [temp,ii1,ii_3p2] = intersect(true_alt_3p,true_splice);
  assert(length(temp)==length(unique(true_alt_3p)))
  true_alt_3p = true_alt_3p(ii1);
  % splice_before_3p = gene.splicegraph{1}(don_end,[exon_alt_3prime(find(idx_alt_3prime==g_idx)).fiveprimesite]);
  % don_before = gene.splicegraph{1}(other_end,[exon_alt_3prime(find(idx_alt_3prime==g_idx)).fiveprimesite]);
  % don_after = gene.splicegraph{1}(other_end,alt_3p);
  label_2(3,ii_3p2) = 1;
  if debug
    fprintf(1,'alt 3prime')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_3p)
      plot([true_alt_3p(jj) true_alt_3p(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard 
    end 
  end
end
[ii,ii1,ii2] = intersect(POS,true_splice);
LABEL_2(:,ii1) = label_2(:,ii2);





return

function ALL = get_coverage(ALL,gene,true_splice,label_2,info,info_names)
% [COV1,COV2] = get_coverage(ALL,gene,true_splice,info_names)
% 
% 
% get sum over all coverage for each true_splicesite and certain transcript()
% 
% INPUT  - gene:  gene structure of length 1;  
%          required fields:transcript_info,transcript_valid,exons,exons_confirmed, is_alt
%        - true_splice_site:    vector of all true splice sites (either
%          donor or acceptor, not both) that occur in the
%          splicegraph of the gene.
%        - info_names: cell which tells, you which transcripts to look
%          at ('anno','cDNA',...)
%   
% OUTPUT - cov1  number of coverage by given transcript type (according to info_names) for each true splice site. 
%        - cov2  number of times skipped by given transcript type (according to info_names) for each true splice site. 

  
COV1 = ALL.COV1 ;
COV2 = ALL.COV2;
POS = ALL.POS;
  
genic_idx = find(POS>gene.start&POS<gene.stop);
genic_pos = POS(genic_idx); 
assert(all(all(COV2(:,genic_idx)==0)))
assert(all(all(COV1(:,genic_idx)==0)))

cov1 = zeros(size(info_names,1),length(true_splice));
cov2 = zeros(size(info_names,1),length(true_splice));

% idx_const = find(sum(label_2==1)==0);
const_exons = gene.splicegraph{1}(:,find(~gene.exon_in_alt_region));
[tmp,idx_const,tmp] = intersect(true_splice,const_exons(:));
assert(all(sum(label_2(:,idx_const)==1)==0))
for type = 1:size(info_names,1) 
  if isequal(info_names{type,1},'anno')
    cov1(type,:) = -inf;
    cov2(type,:) = -inf;
  end
  t_idx = [];
  for tt1 = 1:size(info_names,2)
    if ~isempty(info_names{type,tt1})
      t_idx = [t_idx find(gene.transcript_info==info.(info_names{type,tt1}))];  
    end
  end
  if isempty(t_idx)
    continue
  end
  for t = t_idx
    if ~gene.transcript_valid(t)
      continue
    end
    [cov1_t, cov2_t] = get_coverage4transcript(gene,t,true_splice,info);
    if gene.transcript_info(t)==info.anno
      cov1(type,:) = max(cov1(type,:),cov1_t);
      cov2(type,:) = max(cov2(type,:),cov2_t); 
      if ~isempty(idx_const)&&~all(isinf(cov2_t(idx_const)))
        warning('here seems to be bug that should be fixed ... ') ;
        if keyboard_allowed()
          keyboard
        end
      end
    else
      cov1(type,:) = cov1(type,:) +cov1_t;
      cov2(type,:) = cov2(type,:) +cov2_t;
      if ~isempty(idx_const)&&~all(cov2_t(idx_const)==0)
        if keyboard_allowed()
          keyboard
        end
      end
    end
  end %%% loop over transcripts
end %%% loop over transcript types
idx_anno = find(strcmp(info_names,'anno')) ;
not_anno = setdiff(idx_anno,[1:size(info_names,1)]);

ii_alt2 = find(any(label_2==1));
ii_ir2 = find(label_2(2,:)==1);
if ~isempty(ii_alt2) && ~all(sum(cov2(not_anno,ii_alt2))>0|any(~isinf(cov2(idx_anno,ii_alt2))))
  warning('no cov2')
  % hold off;
  % viewsplicegraph(gene);
  if ~isempty(ii_ir2)
    fprintf('intron retention')
    % for a=1:length(true_alt_ir)
    %    hold on ; plot([true_alt_ir(a) true_alt_ir(a)],[1 10],'b')
    % end
  else
    %keyboard
  end
  gene.id
  warning('keyboard')
  %keyboard
end


[ii,ii1,ii2] = intersect(POS,true_splice);
assert(all(ismember(ii1,genic_idx)))

COV1(:,ii1) = cov1(:,ii2);
COV2(:,ii1) = cov2(:,ii2);

%%% SET COV2 OF NEGATIVE GENIC POSITIONS to min over coverage of
% neigbouring truesplicesites

true_splice = true_splice(ii2);
cov1 = cov1(:,ii2);


genic_neg_pos = setdiff(genic_pos,true_splice);
genic_cov2  =   zeros(size(info_names,1),length(genic_neg_pos));
[temp,ii1,ii2] = intersect(POS,genic_neg_pos);
assert(length(temp)==length(genic_neg_pos));
genic_neg_pos = genic_neg_pos(ii2);
if ~isequal(determine_engine, 'octave'),
  assert(issorted(true_splice))
end ;
for a=2:length(true_splice)
  idx = find(genic_neg_pos<true_splice(a)&genic_neg_pos>true_splice(a-1));
  genic_cov2(:,idx) = repmat(min([cov1(:,a),cov1(:,a-1)]'),length(idx),1)';
end
assert(all(all(COV2(:,ii1)==0)))
assert(all(all(COV1(:,ii1)==0)))
COV2(:,ii1) =genic_cov2;
    






function [cov1, cov2] = get_coverage4transcript(gene,transcript_idx,true_splice,info)

% [cov1, cov2] = get_coverage(gene,transcrit_idx,true_splice)
%
% for a given transcript transcrit_idx of the gene counts number of coverage for each
% of the true splice sites. 
% returned is cov1 the number of times a splice site is used in one
% evidence (e.g. transcript). In the case of ESTs this can be more than
% one, because a single transcript is produced out of a cluster of
% several ESTs.This information is stored in the
% field exons_confirmed{transcript_idx}, which is a Nx2 matrix, with the
% first number giving the confirmation for the exon and the second number
% giving the confirmation for the succeding
% intron. (exons_confirmed{transcript_idx}(end,2) shoeld always be 0, as
% there is no intron after the last exon.)
% The second returned value cov2 gives the number of evidences that
% disagree with the splicesite. e.i. the number of times a splice site is skipped.
   
  

exons = gene.exons{transcript_idx}(:,1:2);
exons_conf = gene.exons_confirmed{transcript_idx};

cov1 = -inf(1,length(true_splice));
cov2 = -inf(1,length(true_splice));

for a = 1:length(true_splice)
  %%% CHECK COVERAGE OF INCLUDED FORM     
  [idx1,idx2] = find(exons==true_splice(a));
  if ~isempty(idx1)
    assert(length(idx1)==1)
    cov1(a) = exons_conf(idx1,1); 
  end
  %%% CHECK EXON COVERAGE OF SKIPPED FORM    
  idx  = find(exons(:,1)<true_splice(a)&exons(:,2)>true_splice(a)) ;
  if ~isempty(idx)   
    assert(length(idx)==1)
    % cov2(a) = exons_conf(idx,2); 
    if gene.transcript_info(transcript_idx)==info.est|gene.transcript_info(transcript_idx)==info.cDNA &&...
          (idx==1|idx==size(exons,1))
      if all(label_2(:,a)==-1)
        continue
      else
        % alternative 3' or 5' or first or last exon; est ends
        % are usually unconfirmed 
      end
    elseif gene.transcript_info(transcript_idx)==info.anno
      if min(abs(true_splice(a)-exons(idx,:)))<3 
        % keyboard
        continue
      end
    end
    cov2(a) = exons_conf(idx,1);
  end  
  %%% CHECK INTRON COVERAGE OF SKIPPED FORM
  idx  = find(exons(1:end-1,2)<true_splice(a)&exons(2:end,1)>true_splice(a)) ;
  if ~isempty(idx)            
    assert(length(idx)==1)           
    if gene.transcript_info(transcript_idx)==info.anno
      cov2(a) = max(cov2(a),exons_conf(idx,2));      
    else
      cov2(a) = cov2(a)+exons_conf(idx,2);
    end
  end 
end %% loop over true_splice sites

assert(~any(~isinf(cov1)&~isinf(cov2)))





return

function flag = segment_overlap(start,stop,starts,stops)
% flag = segment_overlap(start,stop,starts,stops)  

flag = ((start >= starts) & (start<=stops)) | ...
       ((stop >= starts) & (stop<=stops)) | ...
       ((start <=starts) & (stop>=stops)) | ...
       ((start >=starts) & (stop<=stops)) ;
  
return

function [regions,N]=get_connected_regions(bw);
  
 % similar to bw_label just for 1xm matrix 

reg_starts = find(bw(1:end)-[0 bw(1:end-1)]==1);
reg_ends = find(bw(1:end)-[bw(2:end) 0]==1);
assert(length(reg_starts)==length(reg_ends))

N = length(reg_starts);

regions = zeros(size(bw));
for i=1:N
  assert(reg_starts(i)<=reg_ends(i))
  regions(reg_starts(i):reg_ends(i))=i;
end
return

function x=reverse_complement(y)

if length(y)>200*1024*1024
  x=uint8(y) ;
  for i=1:length(y),
    x(i)=y(end-i+1) ;
  end ;
  y=x ;
else
  y=y(end:-1:1) ;
  x=y ;
end ;

x(y=='a')='t' ;
x(y=='c')='g' ;
x(y=='g')='c' ;
x(y=='t')='a' ;
x(y=='n')='n' ;
x(y=='A')='T' ;
x(y=='C')='G' ;
x(y=='G')='C' ;
x(y=='T')='A' ;
x(y=='N')='N' ;

x=char(x) ;

return

function array = load_struct(infile_name, arrayname)
% array = load_struct(infile_name, arrayname)
  
if nargin<2,
  arrayname='array' ;
end ;

L_=load(infile_name) ;
if isfield(L_,'numvars')
  fprintf('arrayload2 loads %i chunks:', L_.numvars) ;
  ix=1;
  L=load(sprintf('%s_%i.mat', infile_name, ix), sprintf('%s%d', arrayname, ix)) ;
  eval(sprintf('array = L.%s1;', arrayname)) ;
  fprintf('.') ;
  for ix = 2:L_.numvars
    L=load(sprintf('%s_%i.mat', infile_name, ix), sprintf('%s%d', arrayname, ix)) ;
    if size(array,1) == 1
      array = [array, eval(sprintf('L.%s%d', arrayname, ix))];
    elseif size(array,2) == 1
      array = [array; eval(sprintf('L.%s%d', arrayname, ix))];
    end
    fprintf('.') ;
  end
  fprintf('\n') ;
  if isfield(L_, 'dimensions'),
    array = reshape(array, L_.dimensions) ;
  end ;
else
  eval(sprintf('array=L_.%s;', arrayname)) ;
end

return

function [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% 
% returns either 'matlab' or 'octave' in variable engine
% and 'internal' or 'galaxy' in variable environment
% basedir is the directory of the matlab or octave instance
% mccdir is the directory of the matlab compiler (does not yet exist for octave)

global g_license_type ;

if isempty(g_license_type),
  g_license_type = 'academic' ;
end ;
license_type = g_license_type ;

lic=license ;
if ~isequal(lic, 'GNU General Public License'),
  engine='matlab' ;
else
  engine='octave' ;
end ;

environment='internal' ;
if isequal(whoami, 'galaxy'),
  environment='galaxy' ;
end ;

if isequal(engine, 'matlab') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(engine, 'matlab') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/matlab-7.6/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/octave-3.0.3/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/octave-3.0.4' ;
end ;

if isequal(environment,  'internal'),
  mccdir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(environment,  'galaxy'),
  mccdir = '/home/galaxy/matlab-7.6/' ;
end ;

return

return

function save_append(fname__, append_flag, varargin)
% save_append(fname__, append_flag, varargin)
%
% octave compatible version of save -append

try,
  if append_flag
    L__=load(fname__) ;
  else
    L__=struct ;
  end ;
catch 
  L__=struct ;
end 

if length(varargin)>1,
  
  % assume varargin to have the following structure: varname1, var1, ...
  
  assert(mod(length(varargin),2)==0) ;
  for i=1:2:length(varargin),
    L__.(varargin{i}) = varargin{i+1} ;
  end ;
  
  f=fieldnames(L__) ;
  for i=1:length(f),
    eval(sprintf('%s=L__.%s;', f{i}, f{i})) ;
  end ;
  
  eval(sprintf('save -V7 %s %s', fname__, sprintf('%s ', f{:}))) ;
else
  
  % assume the second argument is a struct
  assert(length(varargin)==1) ;
  
  f = fieldnames(varargin{1}) ;
  for i=1:2:length(f),
    L__.(f{i}) = varargin{1}.(f{i}) ;
  end ;
  
  f=fieldnames(L__) ;
  for i=1:length(f),
    eval(sprintf('%s=L__.%s;', f{i}, f{i})) ;
  end ;
  
  eval(sprintf('save -V7 %s %s', fname__, sprintf('%s ', f{:}))) ;
  
end ;




return

function ret=keyboard_allowed()
% ret=keyboard_allowed()
%
% returns 1, if a keyboard command would be allowed

global g_ignore_keyboard
global THIS_IS_A_RPROC_PROCESS  

if isequal(g_ignore_keyboard, 1),
  ret=0 ;
  return ;
end ;

if isequal(THIS_IS_A_RPROC_PROCESS, 1),
  ret=0 ;
  return ;
end ;

ret=1 ;


return

